#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example pauses an ad."""


import argparse
import sys
from typing import List

from google.api_core import protobuf_helpers

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
    MutateAdGroupAdsResponse,
)


def main(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    ad_id: str,
) -> None:
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )

    ad_group_ad_operation: AdGroupAdOperation = client.get_type(
        "AdGroupAdOperation"
    )

    ad_group_ad: AdGroupAd = ad_group_ad_operation.update
    ad_group_ad.resource_name = ad_group_ad_service.ad_group_ad_path(
        customer_id, ad_group_id, ad_id
    )
    ad_group_ad.status = client.enums.AdGroupAdStatusEnum.PAUSED
    client.copy_from(
        ad_group_ad_operation.update_mask,
        protobuf_helpers.field_mask(None, ad_group_ad._pb),
    )

    operations: List[AdGroupAdOperation] = [ad_group_ad_operation]

    ad_group_ad_response: MutateAdGroupAdsResponse = (
        ad_group_ad_service.mutate_ad_group_ads(
            customer_id=customer_id,
            operations=operations,
        )
    )

    print(
        f"Paused ad group ad {ad_group_ad_response.results[0].resource_name}."
    )


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=("Pauses an ad in the specified customer's ad group.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a", "--ad_group_id", type=str, required=True, help="The ad group ID."
    )
    parser.add_argument(
        "-i", "--ad_id", type=str, required=True, help="The ad ID."
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.ad_group_id, args.ad_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "ConversionOrAdjustmentLagBucketEnum",
    },
)


class ConversionOrAdjustmentLagBucketEnum(proto.Message):
    r"""Container for enum representing the number of days between
    the impression and the conversion or between the impression and
    adjustments to the conversion.

    """

    class ConversionOrAdjustmentLagBucket(proto.Enum):
        r"""Enum representing the number of days between the impression
        and the conversion or between the impression and adjustments to
        the conversion.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CONVERSION_LESS_THAN_ONE_DAY (2):
                Conversion lag bucket from 0 to 1 day. 0 day
                is included, 1 day is not.
            CONVERSION_ONE_TO_TWO_DAYS (3):
                Conversion lag bucket from 1 to 2 days. 1 day
                is included, 2 days is not.
            CONVERSION_TWO_TO_THREE_DAYS (4):
                Conversion lag bucket from 2 to 3 days. 2
                days is included, 3 days is not.
            CONVERSION_THREE_TO_FOUR_DAYS (5):
                Conversion lag bucket from 3 to 4 days. 3
                days is included, 4 days is not.
            CONVERSION_FOUR_TO_FIVE_DAYS (6):
                Conversion lag bucket from 4 to 5 days. 4
                days is included, 5 days is not.
            CONVERSION_FIVE_TO_SIX_DAYS (7):
                Conversion lag bucket from 5 to 6 days. 5
                days is included, 6 days is not.
            CONVERSION_SIX_TO_SEVEN_DAYS (8):
                Conversion lag bucket from 6 to 7 days. 6
                days is included, 7 days is not.
            CONVERSION_SEVEN_TO_EIGHT_DAYS (9):
                Conversion lag bucket from 7 to 8 days. 7
                days is included, 8 days is not.
            CONVERSION_EIGHT_TO_NINE_DAYS (10):
                Conversion lag bucket from 8 to 9 days. 8
                days is included, 9 days is not.
            CONVERSION_NINE_TO_TEN_DAYS (11):
                Conversion lag bucket from 9 to 10 days. 9
                days is included, 10 days is not.
            CONVERSION_TEN_TO_ELEVEN_DAYS (12):
                Conversion lag bucket from 10 to 11 days. 10
                days is included, 11 days is not.
            CONVERSION_ELEVEN_TO_TWELVE_DAYS (13):
                Conversion lag bucket from 11 to 12 days. 11
                days is included, 12 days is not.
            CONVERSION_TWELVE_TO_THIRTEEN_DAYS (14):
                Conversion lag bucket from 12 to 13 days. 12
                days is included, 13 days is not.
            CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS (15):
                Conversion lag bucket from 13 to 14 days. 13
                days is included, 14 days is not.
            CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS (16):
                Conversion lag bucket from 14 to 21 days. 14
                days is included, 21 days is not.
            CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS (17):
                Conversion lag bucket from 21 to 30 days. 21
                days is included, 30 days is not.
            CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS (18):
                Conversion lag bucket from 30 to 45 days. 30
                days is included, 45 days is not.
            CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS (19):
                Conversion lag bucket from 45 to 60 days. 45
                days is included, 60 days is not.
            CONVERSION_SIXTY_TO_NINETY_DAYS (20):
                Conversion lag bucket from 60 to 90 days. 60
                days is included, 90 days is not.
            ADJUSTMENT_LESS_THAN_ONE_DAY (21):
                Conversion adjustment lag bucket from 0 to 1
                day. 0 day is included, 1 day is not.
            ADJUSTMENT_ONE_TO_TWO_DAYS (22):
                Conversion adjustment lag bucket from 1 to 2
                days. 1 day is included, 2 days is not.
            ADJUSTMENT_TWO_TO_THREE_DAYS (23):
                Conversion adjustment lag bucket from 2 to 3
                days. 2 days is included, 3 days is not.
            ADJUSTMENT_THREE_TO_FOUR_DAYS (24):
                Conversion adjustment lag bucket from 3 to 4
                days. 3 days is included, 4 days is not.
            ADJUSTMENT_FOUR_TO_FIVE_DAYS (25):
                Conversion adjustment lag bucket from 4 to 5
                days. 4 days is included, 5 days is not.
            ADJUSTMENT_FIVE_TO_SIX_DAYS (26):
                Conversion adjustment lag bucket from 5 to 6
                days. 5 days is included, 6 days is not.
            ADJUSTMENT_SIX_TO_SEVEN_DAYS (27):
                Conversion adjustment lag bucket from 6 to 7
                days. 6 days is included, 7 days is not.
            ADJUSTMENT_SEVEN_TO_EIGHT_DAYS (28):
                Conversion adjustment lag bucket from 7 to 8
                days. 7 days is included, 8 days is not.
            ADJUSTMENT_EIGHT_TO_NINE_DAYS (29):
                Conversion adjustment lag bucket from 8 to 9
                days. 8 days is included, 9 days is not.
            ADJUSTMENT_NINE_TO_TEN_DAYS (30):
                Conversion adjustment lag bucket from 9 to 10
                days. 9 days is included, 10 days is not.
            ADJUSTMENT_TEN_TO_ELEVEN_DAYS (31):
                Conversion adjustment lag bucket from 10 to
                11 days. 10 days is included, 11 days is not.
            ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS (32):
                Conversion adjustment lag bucket from 11 to
                12 days. 11 days is included, 12 days is not.
            ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS (33):
                Conversion adjustment lag bucket from 12 to
                13 days. 12 days is included, 13 days is not.
            ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS (34):
                Conversion adjustment lag bucket from 13 to
                14 days. 13 days is included, 14 days is not.
            ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS (35):
                Conversion adjustment lag bucket from 14 to
                21 days. 14 days is included, 21 days is not.
            ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS (36):
                Conversion adjustment lag bucket from 21 to
                30 days. 21 days is included, 30 days is not.
            ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS (37):
                Conversion adjustment lag bucket from 30 to
                45 days. 30 days is included, 45 days is not.
            ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS (38):
                Conversion adjustment lag bucket from 45 to
                60 days. 45 days is included, 60 days is not.
            ADJUSTMENT_SIXTY_TO_NINETY_DAYS (39):
                Conversion adjustment lag bucket from 60 to
                90 days. 60 days is included, 90 days is not.
            ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS (40):
                Conversion adjustment lag bucket from 90 to
                145 days. 90 days is included, 145 days is not.
            CONVERSION_UNKNOWN (41):
                Conversion lag bucket UNKNOWN. This is for
                dates before conversion lag bucket was available
                in Google Ads.
            ADJUSTMENT_UNKNOWN (42):
                Conversion adjustment lag bucket UNKNOWN.
                This is for dates before conversion adjustment
                lag bucket was available in Google Ads.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONVERSION_LESS_THAN_ONE_DAY = 2
        CONVERSION_ONE_TO_TWO_DAYS = 3
        CONVERSION_TWO_TO_THREE_DAYS = 4
        CONVERSION_THREE_TO_FOUR_DAYS = 5
        CONVERSION_FOUR_TO_FIVE_DAYS = 6
        CONVERSION_FIVE_TO_SIX_DAYS = 7
        CONVERSION_SIX_TO_SEVEN_DAYS = 8
        CONVERSION_SEVEN_TO_EIGHT_DAYS = 9
        CONVERSION_EIGHT_TO_NINE_DAYS = 10
        CONVERSION_NINE_TO_TEN_DAYS = 11
        CONVERSION_TEN_TO_ELEVEN_DAYS = 12
        CONVERSION_ELEVEN_TO_TWELVE_DAYS = 13
        CONVERSION_TWELVE_TO_THIRTEEN_DAYS = 14
        CONVERSION_THIRTEEN_TO_FOURTEEN_DAYS = 15
        CONVERSION_FOURTEEN_TO_TWENTY_ONE_DAYS = 16
        CONVERSION_TWENTY_ONE_TO_THIRTY_DAYS = 17
        CONVERSION_THIRTY_TO_FORTY_FIVE_DAYS = 18
        CONVERSION_FORTY_FIVE_TO_SIXTY_DAYS = 19
        CONVERSION_SIXTY_TO_NINETY_DAYS = 20
        ADJUSTMENT_LESS_THAN_ONE_DAY = 21
        ADJUSTMENT_ONE_TO_TWO_DAYS = 22
        ADJUSTMENT_TWO_TO_THREE_DAYS = 23
        ADJUSTMENT_THREE_TO_FOUR_DAYS = 24
        ADJUSTMENT_FOUR_TO_FIVE_DAYS = 25
        ADJUSTMENT_FIVE_TO_SIX_DAYS = 26
        ADJUSTMENT_SIX_TO_SEVEN_DAYS = 27
        ADJUSTMENT_SEVEN_TO_EIGHT_DAYS = 28
        ADJUSTMENT_EIGHT_TO_NINE_DAYS = 29
        ADJUSTMENT_NINE_TO_TEN_DAYS = 30
        ADJUSTMENT_TEN_TO_ELEVEN_DAYS = 31
        ADJUSTMENT_ELEVEN_TO_TWELVE_DAYS = 32
        ADJUSTMENT_TWELVE_TO_THIRTEEN_DAYS = 33
        ADJUSTMENT_THIRTEEN_TO_FOURTEEN_DAYS = 34
        ADJUSTMENT_FOURTEEN_TO_TWENTY_ONE_DAYS = 35
        ADJUSTMENT_TWENTY_ONE_TO_THIRTY_DAYS = 36
        ADJUSTMENT_THIRTY_TO_FORTY_FIVE_DAYS = 37
        ADJUSTMENT_FORTY_FIVE_TO_SIXTY_DAYS = 38
        ADJUSTMENT_SIXTY_TO_NINETY_DAYS = 39
        ADJUSTMENT_NINETY_TO_ONE_HUNDRED_AND_FORTY_FIVE_DAYS = 40
        CONVERSION_UNKNOWN = 41
        ADJUSTMENT_UNKNOWN = 42


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.enums",
    marshal="google.ads.googleads.v20",
    manifest={
        "MimeTypeEnum",
    },
)


class MimeTypeEnum(proto.Message):
    r"""Container for enum describing the mime types."""

    class MimeType(proto.Enum):
        r"""The mime type

        Values:
            UNSPECIFIED (0):
                The mime type has not been specified.
            UNKNOWN (1):
                The received value is not known in this
                version.
                This is a response-only value.
            IMAGE_JPEG (2):
                MIME type of image/jpeg.
            IMAGE_GIF (3):
                MIME type of image/gif.
            IMAGE_PNG (4):
                MIME type of image/png.
            FLASH (5):
                MIME type of application/x-shockwave-flash.
            TEXT_HTML (6):
                MIME type of text/html.
            PDF (7):
                MIME type of application/pdf.
            MSWORD (8):
                MIME type of application/msword.
            MSEXCEL (9):
                MIME type of application/vnd.ms-excel.
            RTF (10):
                MIME type of application/rtf.
            AUDIO_WAV (11):
                MIME type of audio/wav.
            AUDIO_MP3 (12):
                MIME type of audio/mp3.
            HTML5_AD_ZIP (13):
                MIME type of application/x-html5-ad-zip.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        IMAGE_JPEG = 2
        IMAGE_GIF = 3
        IMAGE_PNG = 4
        FLASH = 5
        TEXT_HTML = 6
        PDF = 7
        MSWORD = 8
        MSEXCEL = 9
        RTF = 10
        AUDIO_WAV = 11
        AUDIO_MP3 = 12
        HTML5_AD_ZIP = 13


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "ResourceLimitTypeEnum",
    },
)


class ResourceLimitTypeEnum(proto.Message):
    r"""Container for enum describing possible resource limit types."""

    class ResourceLimitType(proto.Enum):
        r"""Resource limit type.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents an
                unclassified operation unknown in this version.
            CAMPAIGNS_PER_CUSTOMER (2):
                Number of ENABLED and PAUSED campaigns per
                customer.
            BASE_CAMPAIGNS_PER_CUSTOMER (3):
                Number of ENABLED and PAUSED base campaigns
                per customer.
            EXPERIMENT_CAMPAIGNS_PER_CUSTOMER (105):
                Number of ENABLED and PAUSED experiment
                campaigns per customer.
            HOTEL_CAMPAIGNS_PER_CUSTOMER (4):
                Number of ENABLED and PAUSED Hotel campaigns
                per customer.
            SMART_SHOPPING_CAMPAIGNS_PER_CUSTOMER (5):
                Number of ENABLED and PAUSED Smart Shopping
                campaigns per customer.
            AD_GROUPS_PER_CAMPAIGN (6):
                Number of ENABLED ad groups per campaign.
            AD_GROUPS_PER_SHOPPING_CAMPAIGN (8):
                Number of ENABLED ad groups per Shopping
                campaign.
            AD_GROUPS_PER_HOTEL_CAMPAIGN (9):
                Number of ENABLED ad groups per Hotel
                campaign.
            REPORTING_AD_GROUPS_PER_LOCAL_CAMPAIGN (10):
                Number of ENABLED reporting ad groups per
                local campaign.
            REPORTING_AD_GROUPS_PER_APP_CAMPAIGN (11):
                Number of ENABLED reporting ad groups per App
                campaign. It includes app campaign and app
                campaign for engagement.
            MANAGED_AD_GROUPS_PER_SMART_CAMPAIGN (52):
                Number of ENABLED managed ad groups per smart
                campaign.
            AD_GROUP_CRITERIA_PER_CUSTOMER (12):
                Number of ENABLED ad group criteria per
                customer. An ad group criterion is considered as
                ENABLED if:

                1. it's not REMOVED
                2. its ad group is not REMOVED
                3. its campaign is not REMOVED.
            BASE_AD_GROUP_CRITERIA_PER_CUSTOMER (13):
                Number of ad group criteria across all base
                campaigns for a customer.
            EXPERIMENT_AD_GROUP_CRITERIA_PER_CUSTOMER (107):
                Number of ad group criteria across all
                experiment campaigns for a customer.
            AD_GROUP_CRITERIA_PER_CAMPAIGN (14):
                Number of ENABLED ad group criteria per
                campaign. An ad group criterion is considered as
                ENABLED if:

                1. it's not REMOVED
                2. its ad group is not REMOVED.
            CAMPAIGN_CRITERIA_PER_CUSTOMER (15):
                Number of ENABLED campaign criteria per
                customer.
            BASE_CAMPAIGN_CRITERIA_PER_CUSTOMER (16):
                Number of ENABLED campaign criteria across
                all base campaigns for a customer.
            EXPERIMENT_CAMPAIGN_CRITERIA_PER_CUSTOMER (108):
                Number of ENABLED campaign criteria across
                all experiment campaigns for a customer.
            WEBPAGE_CRITERIA_PER_CUSTOMER (17):
                Number of ENABLED webpage criteria per
                customer, including campaign level and ad group
                level.
            BASE_WEBPAGE_CRITERIA_PER_CUSTOMER (18):
                Number of ENABLED webpage criteria across all
                base campaigns for a customer.
            EXPERIMENT_WEBPAGE_CRITERIA_PER_CUSTOMER (19):
                Meximum number of ENABLED webpage criteria
                across all experiment campaigns for a customer.
            COMBINED_AUDIENCE_CRITERIA_PER_AD_GROUP (20):
                Number of combined audience criteria per ad
                group.
            CUSTOMER_NEGATIVE_PLACEMENT_CRITERIA_PER_CUSTOMER (21):
                Limit for placement criterion type group in
                customer negative criterion.
            CUSTOMER_NEGATIVE_YOUTUBE_CHANNEL_CRITERIA_PER_CUSTOMER (22):
                Limit for YouTube TV channels in customer
                negative criterion.
            CRITERIA_PER_AD_GROUP (23):
                Number of ENABLED criteria per ad group.
            LISTING_GROUPS_PER_AD_GROUP (24):
                Number of listing group criteria per ad
                group.
            EXPLICITLY_SHARED_BUDGETS_PER_CUSTOMER (25):
                Number of ENABLED explicitly shared budgets
                per customer.
            IMPLICITLY_SHARED_BUDGETS_PER_CUSTOMER (26):
                Number of ENABLED implicitly shared budgets
                per customer.
            COMBINED_AUDIENCE_CRITERIA_PER_CAMPAIGN (27):
                Number of combined audience criteria per
                campaign.
            NEGATIVE_KEYWORDS_PER_CAMPAIGN (28):
                Number of negative keywords per campaign.
            NEGATIVE_PLACEMENTS_PER_CAMPAIGN (29):
                Number of excluded campaign criteria in
                placement dimension, for example, placement,
                mobile application, YouTube channel, etc. The
                API criterion type is NOT limited to placement
                only, and this does not include exclusions at
                the ad group or other levels.
            GEO_TARGETS_PER_CAMPAIGN (30):
                Number of geo targets per campaign.
            NEGATIVE_IP_BLOCKS_PER_CAMPAIGN (32):
                Number of negative IP blocks per campaign.
            PROXIMITIES_PER_CAMPAIGN (33):
                Number of proximity targets per campaign.
            LISTING_SCOPES_PER_SHOPPING_CAMPAIGN (34):
                Number of listing scopes per Shopping
                campaign.
            LISTING_SCOPES_PER_NON_SHOPPING_CAMPAIGN (35):
                Number of listing scopes per non-Shopping
                campaign.
            NEGATIVE_KEYWORDS_PER_SHARED_SET (36):
                Number of criteria per negative keyword
                shared set.
            NEGATIVE_PLACEMENTS_PER_SHARED_SET (37):
                Number of criteria per negative placement
                shared set.
            SHARED_SETS_PER_CUSTOMER_FOR_TYPE_DEFAULT (40):
                Default number of shared sets allowed per
                type per customer.
            SHARED_SETS_PER_CUSTOMER_FOR_NEGATIVE_PLACEMENT_LIST_LOWER (41):
                Number of shared sets of negative placement
                list type for a manager customer.
            HOTEL_ADVANCE_BOOKING_WINDOW_BID_MODIFIERS_PER_AD_GROUP (44):
                Number of hotel_advance_booking_window bid modifiers per ad
                group.
            BIDDING_STRATEGIES_PER_CUSTOMER (45):
                Number of ENABLED shared bidding strategies
                per customer.
            BASIC_USER_LISTS_PER_CUSTOMER (47):
                Number of open basic user lists per customer.
            LOGICAL_USER_LISTS_PER_CUSTOMER (48):
                Number of open logical user lists per
                customer.
            RULE_BASED_USER_LISTS_PER_CUSTOMER (153):
                Number of open rule based user lists per
                customer.
            BASE_AD_GROUP_ADS_PER_CUSTOMER (53):
                Number of ENABLED and PAUSED ad group ads
                across all base campaigns for a customer.
            EXPERIMENT_AD_GROUP_ADS_PER_CUSTOMER (54):
                Number of ENABLED and PAUSED ad group ads
                across all experiment campaigns for a customer.
            AD_GROUP_ADS_PER_CAMPAIGN (55):
                Number of ENABLED and PAUSED ad group ads per
                campaign.
            TEXT_AND_OTHER_ADS_PER_AD_GROUP (56):
                Number of ENABLED ads per ad group that do
                not fall in to other buckets. Includes text and
                many other types.
            IMAGE_ADS_PER_AD_GROUP (57):
                Number of ENABLED image ads per ad group.
            SHOPPING_SMART_ADS_PER_AD_GROUP (58):
                Number of ENABLED shopping smart ads per ad
                group.
            RESPONSIVE_SEARCH_ADS_PER_AD_GROUP (59):
                Number of ENABLED responsive search ads per
                ad group.
            APP_ADS_PER_AD_GROUP (60):
                Number of ENABLED app ads per ad group.
            APP_ENGAGEMENT_ADS_PER_AD_GROUP (61):
                Number of ENABLED app engagement ads per ad
                group.
            LOCAL_ADS_PER_AD_GROUP (62):
                Number of ENABLED local ads per ad group.
            VIDEO_ADS_PER_AD_GROUP (63):
                Number of ENABLED video ads per ad group.
            LEAD_FORM_CAMPAIGN_ASSETS_PER_CAMPAIGN (143):
                Number of ENABLED lead form CampaignAssets
                per campaign.
            PROMOTION_CUSTOMER_ASSETS_PER_CUSTOMER (79):
                Number of ENABLED promotion CustomerAssets
                per customer.
            PROMOTION_CAMPAIGN_ASSETS_PER_CAMPAIGN (80):
                Number of ENABLED promotion CampaignAssets
                per campaign.
            PROMOTION_AD_GROUP_ASSETS_PER_AD_GROUP (81):
                Number of ENABLED promotion AdGroupAssets per
                ad group.
            CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER (134):
                Number of ENABLED callout CustomerAssets per
                customer.
            CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN (135):
                Number of ENABLED callout CampaignAssets per
                campaign.
            CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP (136):
                Number of ENABLED callout AdGroupAssets per
                ad group.
            SITELINK_CUSTOMER_ASSETS_PER_CUSTOMER (137):
                Number of ENABLED sitelink CustomerAssets per
                customer.
            SITELINK_CAMPAIGN_ASSETS_PER_CAMPAIGN (138):
                Number of ENABLED sitelink CampaignAssets per
                campaign.
            SITELINK_AD_GROUP_ASSETS_PER_AD_GROUP (139):
                Number of ENABLED sitelink AdGroupAssets per
                ad group.
            STRUCTURED_SNIPPET_CUSTOMER_ASSETS_PER_CUSTOMER (140):
                Number of ENABLED structured snippet
                CustomerAssets per customer.
            STRUCTURED_SNIPPET_CAMPAIGN_ASSETS_PER_CAMPAIGN (141):
                Number of ENABLED structured snippet
                CampaignAssets per campaign.
            STRUCTURED_SNIPPET_AD_GROUP_ASSETS_PER_AD_GROUP (142):
                Number of ENABLED structured snippet
                AdGroupAssets per ad group.
            MOBILE_APP_CUSTOMER_ASSETS_PER_CUSTOMER (144):
                Number of ENABLED mobile app CustomerAssets
                per customer.
            MOBILE_APP_CAMPAIGN_ASSETS_PER_CAMPAIGN (145):
                Number of ENABLED mobile app CampaignAssets
                per campaign.
            MOBILE_APP_AD_GROUP_ASSETS_PER_AD_GROUP (146):
                Number of ENABLED mobile app AdGroupAssets
                per ad group.
            HOTEL_CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER (147):
                Number of ENABLED hotel callout
                CustomerAssets per customer.
            HOTEL_CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN (148):
                Number of ENABLED hotel callout
                CampaignAssets per campaign.
            HOTEL_CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP (149):
                Number of ENABLED hotel callout AdGroupAssets
                per ad group.
            CALL_CUSTOMER_ASSETS_PER_CUSTOMER (150):
                Number of ENABLED call CustomerAssets per
                customer.
            CALL_CAMPAIGN_ASSETS_PER_CAMPAIGN (151):
                Number of ENABLED call CampaignAssets per
                campaign.
            CALL_AD_GROUP_ASSETS_PER_AD_GROUP (152):
                Number of ENABLED call AdGroupAssets per ad
                group.
            PRICE_CUSTOMER_ASSETS_PER_CUSTOMER (154):
                Number of ENABLED price CustomerAssets per
                customer.
            PRICE_CAMPAIGN_ASSETS_PER_CAMPAIGN (155):
                Number of ENABLED price CampaignAssets per
                campaign.
            PRICE_AD_GROUP_ASSETS_PER_AD_GROUP (156):
                Number of ENABLED price AdGroupAssets per ad
                group.
            AD_IMAGE_CAMPAIGN_ASSETS_PER_CAMPAIGN (175):
                Number of ENABLED ad image CampaignAssets per
                campaign.
            AD_IMAGE_AD_GROUP_ASSETS_PER_AD_GROUP (176):
                Number of ENABLED ad image AdGroupAssets per
                ad group.
            PAGE_FEED_ASSET_SETS_PER_CUSTOMER (157):
                Number of ENABLED page feed asset sets per
                customer.
            DYNAMIC_EDUCATION_FEED_ASSET_SETS_PER_CUSTOMER (158):
                Number of ENABLED dynamic education feed
                asset sets per customer.
            ASSETS_PER_PAGE_FEED_ASSET_SET (159):
                Number of ENABLED assets per page feed asset
                set.
            ASSETS_PER_DYNAMIC_EDUCATION_FEED_ASSET_SET (160):
                Number of ENABLED assets per dynamic
                education asset set.
            DYNAMIC_REAL_ESTATE_ASSET_SETS_PER_CUSTOMER (161):
                Number of ENABLED dynamic real estate asset
                sets per customer.
            ASSETS_PER_DYNAMIC_REAL_ESTATE_ASSET_SET (162):
                Number of ENABLED assets per dynamic real
                estate asset set.
            DYNAMIC_CUSTOM_ASSET_SETS_PER_CUSTOMER (163):
                Number of ENABLED dynamic custom asset sets
                per customer.
            ASSETS_PER_DYNAMIC_CUSTOM_ASSET_SET (164):
                Number of ENABLED assets per dynamic custom
                asset set.
            DYNAMIC_HOTELS_AND_RENTALS_ASSET_SETS_PER_CUSTOMER (165):
                Number of ENABLED dynamic hotels and rentals
                asset sets per customer.
            ASSETS_PER_DYNAMIC_HOTELS_AND_RENTALS_ASSET_SET (166):
                Number of ENABLED assets per dynamic hotels
                and rentals asset set.
            DYNAMIC_LOCAL_ASSET_SETS_PER_CUSTOMER (167):
                Number of ENABLED dynamic local asset sets
                per customer.
            ASSETS_PER_DYNAMIC_LOCAL_ASSET_SET (168):
                Number of ENABLED assets per dynamic local
                asset set.
            DYNAMIC_FLIGHTS_ASSET_SETS_PER_CUSTOMER (169):
                Number of ENABLED dynamic flights asset sets
                per customer.
            ASSETS_PER_DYNAMIC_FLIGHTS_ASSET_SET (170):
                Number of ENABLED assets per dynamic flights
                asset set.
            DYNAMIC_TRAVEL_ASSET_SETS_PER_CUSTOMER (171):
                Number of ENABLED dynamic travel asset sets
                per customer.
            ASSETS_PER_DYNAMIC_TRAVEL_ASSET_SET (172):
                Number of ENABLED assets per dynamic travel
                asset set.
            DYNAMIC_JOBS_ASSET_SETS_PER_CUSTOMER (173):
                Number of ENABLED dynamic jobs asset sets per
                customer.
            ASSETS_PER_DYNAMIC_JOBS_ASSET_SET (174):
                Number of ENABLED assets per dynamic jobs
                asset set.
            BUSINESS_NAME_CAMPAIGN_ASSETS_PER_CAMPAIGN (179):
                Number of ENABLED business name
                CampaignAssets per campaign.
            BUSINESS_LOGO_CAMPAIGN_ASSETS_PER_CAMPAIGN (180):
                Number of ENABLED business logo
                CampaignAssets per campaign.
            VERSIONS_PER_AD (82):
                Number of versions per ad.
            USER_FEEDS_PER_CUSTOMER (90):
                Number of ENABLED user feeds per customer.
            SYSTEM_FEEDS_PER_CUSTOMER (91):
                Number of ENABLED system feeds per customer.
            FEED_ATTRIBUTES_PER_FEED (92):
                Number of feed attributes per feed.
            FEED_ITEMS_PER_CUSTOMER (94):
                Number of ENABLED feed items per customer.
            CAMPAIGN_FEEDS_PER_CUSTOMER (95):
                Number of ENABLED campaign feeds per
                customer.
            BASE_CAMPAIGN_FEEDS_PER_CUSTOMER (96):
                Number of ENABLED campaign feeds across all
                base campaigns for a customer.
            EXPERIMENT_CAMPAIGN_FEEDS_PER_CUSTOMER (109):
                Number of ENABLED campaign feeds across all
                experiment campaigns for a customer.
            AD_GROUP_FEEDS_PER_CUSTOMER (97):
                Number of ENABLED ad group feeds per
                customer.
            BASE_AD_GROUP_FEEDS_PER_CUSTOMER (98):
                Number of ENABLED ad group feeds across all
                base campaigns for a customer.
            EXPERIMENT_AD_GROUP_FEEDS_PER_CUSTOMER (110):
                Number of ENABLED ad group feeds across all
                experiment campaigns for a customer.
            AD_GROUP_FEEDS_PER_CAMPAIGN (99):
                Number of ENABLED ad group feeds per
                campaign.
            FEED_ITEM_SETS_PER_CUSTOMER (100):
                Number of ENABLED feed items per customer.
            FEED_ITEMS_PER_FEED_ITEM_SET (101):
                Number of feed items per feed item set.
            CAMPAIGN_EXPERIMENTS_PER_CUSTOMER (112):
                Number of ENABLED campaign experiments per
                customer.
            EXPERIMENT_ARMS_PER_VIDEO_EXPERIMENT (113):
                Number of video experiment arms per
                experiment.
            OWNED_LABELS_PER_CUSTOMER (115):
                Number of owned labels per customer.
            LABELS_PER_CAMPAIGN (117):
                Number of applied labels per campaign.
            LABELS_PER_AD_GROUP (118):
                Number of applied labels per ad group.
            LABELS_PER_AD_GROUP_AD (119):
                Number of applied labels per ad group ad.
            LABELS_PER_AD_GROUP_CRITERION (120):
                Number of applied labels per ad group
                criterion.
            TARGET_CUSTOMERS_PER_LABEL (121):
                Number of customers with a single label
                applied.
            KEYWORD_PLANS_PER_USER_PER_CUSTOMER (122):
                Number of ENABLED keyword plans per user per
                customer. The limit is applied per <user,
                customer> pair because by default a plan is
                private to a user of a customer. Each user of a
                customer has their own independent limit.
            KEYWORD_PLAN_AD_GROUP_KEYWORDS_PER_KEYWORD_PLAN (123):
                Number of keyword plan ad group keywords per
                keyword plan.
            KEYWORD_PLAN_AD_GROUPS_PER_KEYWORD_PLAN (124):
                Number of keyword plan ad groups per keyword
                plan.
            KEYWORD_PLAN_NEGATIVE_KEYWORDS_PER_KEYWORD_PLAN (125):
                Number of keyword plan negative keywords
                (both campaign and ad group) per keyword plan.
            KEYWORD_PLAN_CAMPAIGNS_PER_KEYWORD_PLAN (126):
                Number of keyword plan campaigns per keyword
                plan.
            CONVERSION_ACTIONS_PER_CUSTOMER (128):
                Number of ENABLED conversion actions per
                customer.
            BATCH_JOB_OPERATIONS_PER_JOB (130):
                Number of operations in a single batch job.
            BATCH_JOBS_PER_CUSTOMER (131):
                Number of PENDING or ENABLED batch jobs per
                customer.
            HOTEL_CHECK_IN_DATE_RANGE_BID_MODIFIERS_PER_AD_GROUP (132):
                Number of hotel check-in date range bid
                modifiers per ad agroup.
            SHARED_SETS_PER_ACCOUNT_FOR_ACCOUNT_LEVEL_NEGATIVE_KEYWORDS (177):
                Number of shared sets of type
                ACCOUNT_LEVEL_NEGATIVE_KEYWORDS per account.
            ACCOUNT_LEVEL_NEGATIVE_KEYWORDS_PER_SHARED_SET (178):
                Number of keywords per ACCOUNT_LEVEL_NEGATIVE_KEYWORDS
                shared set.
            ENABLED_ASSET_PER_HOTEL_PROPERTY_ASSET_SET (181):
                Maximum number of asset per hotel property
                asset set.
            ENABLED_HOTEL_PROPERTY_ASSET_LINKS_PER_ASSET_GROUP (182):
                Maximum number of enabled hotel property
                assets per asset group.
            BRANDS_PER_SHARED_SET (183):
                Number of criteria per brand shared set.
            ENABLED_BRAND_LIST_CRITERIA_PER_CAMPAIGN (184):
                Number of active brand list criteria per
                campaign.
            SHARED_SETS_PER_ACCOUNT_FOR_BRAND (185):
                Maximum number of shared sets of brand type
                for an account.
            LOOKALIKE_USER_LISTS_PER_CUSTOMER (186):
                Maximum number of lookalike lists per
                customer.
            LOGO_CAMPAIGN_ASSETS_PER_CAMPAIGN (187):
                Total number of enabled IMAGE CampaignAssets with LOGO and
                LANDSCAPE_LOGO field types per campaign.
            BUSINESS_MESSAGE_ASSET_LINKS_PER_CUSTOMER (188):
                Maximum number of active business message
                asset links at customer level.
            WHATSAPP_BUSINESS_MESSAGE_ASSET_LINKS_PER_CAMPAIGN (189):
                Maximum number of active WhatsApp business
                message asset links at campaign level.
            WHATSAPP_BUSINESS_MESSAGE_ASSET_LINKS_PER_AD_GROUP (190):
                Maximum number of active WhatsApp business
                message asset links at ad group level.
            BRAND_LIST_CRITERIA_PER_AD_GROUP (193):
                Number of ENABLED brand list criteria per ad
                group.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CAMPAIGNS_PER_CUSTOMER = 2
        BASE_CAMPAIGNS_PER_CUSTOMER = 3
        EXPERIMENT_CAMPAIGNS_PER_CUSTOMER = 105
        HOTEL_CAMPAIGNS_PER_CUSTOMER = 4
        SMART_SHOPPING_CAMPAIGNS_PER_CUSTOMER = 5
        AD_GROUPS_PER_CAMPAIGN = 6
        AD_GROUPS_PER_SHOPPING_CAMPAIGN = 8
        AD_GROUPS_PER_HOTEL_CAMPAIGN = 9
        REPORTING_AD_GROUPS_PER_LOCAL_CAMPAIGN = 10
        REPORTING_AD_GROUPS_PER_APP_CAMPAIGN = 11
        MANAGED_AD_GROUPS_PER_SMART_CAMPAIGN = 52
        AD_GROUP_CRITERIA_PER_CUSTOMER = 12
        BASE_AD_GROUP_CRITERIA_PER_CUSTOMER = 13
        EXPERIMENT_AD_GROUP_CRITERIA_PER_CUSTOMER = 107
        AD_GROUP_CRITERIA_PER_CAMPAIGN = 14
        CAMPAIGN_CRITERIA_PER_CUSTOMER = 15
        BASE_CAMPAIGN_CRITERIA_PER_CUSTOMER = 16
        EXPERIMENT_CAMPAIGN_CRITERIA_PER_CUSTOMER = 108
        WEBPAGE_CRITERIA_PER_CUSTOMER = 17
        BASE_WEBPAGE_CRITERIA_PER_CUSTOMER = 18
        EXPERIMENT_WEBPAGE_CRITERIA_PER_CUSTOMER = 19
        COMBINED_AUDIENCE_CRITERIA_PER_AD_GROUP = 20
        CUSTOMER_NEGATIVE_PLACEMENT_CRITERIA_PER_CUSTOMER = 21
        CUSTOMER_NEGATIVE_YOUTUBE_CHANNEL_CRITERIA_PER_CUSTOMER = 22
        CRITERIA_PER_AD_GROUP = 23
        LISTING_GROUPS_PER_AD_GROUP = 24
        EXPLICITLY_SHARED_BUDGETS_PER_CUSTOMER = 25
        IMPLICITLY_SHARED_BUDGETS_PER_CUSTOMER = 26
        COMBINED_AUDIENCE_CRITERIA_PER_CAMPAIGN = 27
        NEGATIVE_KEYWORDS_PER_CAMPAIGN = 28
        NEGATIVE_PLACEMENTS_PER_CAMPAIGN = 29
        GEO_TARGETS_PER_CAMPAIGN = 30
        NEGATIVE_IP_BLOCKS_PER_CAMPAIGN = 32
        PROXIMITIES_PER_CAMPAIGN = 33
        LISTING_SCOPES_PER_SHOPPING_CAMPAIGN = 34
        LISTING_SCOPES_PER_NON_SHOPPING_CAMPAIGN = 35
        NEGATIVE_KEYWORDS_PER_SHARED_SET = 36
        NEGATIVE_PLACEMENTS_PER_SHARED_SET = 37
        SHARED_SETS_PER_CUSTOMER_FOR_TYPE_DEFAULT = 40
        SHARED_SETS_PER_CUSTOMER_FOR_NEGATIVE_PLACEMENT_LIST_LOWER = 41
        HOTEL_ADVANCE_BOOKING_WINDOW_BID_MODIFIERS_PER_AD_GROUP = 44
        BIDDING_STRATEGIES_PER_CUSTOMER = 45
        BASIC_USER_LISTS_PER_CUSTOMER = 47
        LOGICAL_USER_LISTS_PER_CUSTOMER = 48
        RULE_BASED_USER_LISTS_PER_CUSTOMER = 153
        BASE_AD_GROUP_ADS_PER_CUSTOMER = 53
        EXPERIMENT_AD_GROUP_ADS_PER_CUSTOMER = 54
        AD_GROUP_ADS_PER_CAMPAIGN = 55
        TEXT_AND_OTHER_ADS_PER_AD_GROUP = 56
        IMAGE_ADS_PER_AD_GROUP = 57
        SHOPPING_SMART_ADS_PER_AD_GROUP = 58
        RESPONSIVE_SEARCH_ADS_PER_AD_GROUP = 59
        APP_ADS_PER_AD_GROUP = 60
        APP_ENGAGEMENT_ADS_PER_AD_GROUP = 61
        LOCAL_ADS_PER_AD_GROUP = 62
        VIDEO_ADS_PER_AD_GROUP = 63
        LEAD_FORM_CAMPAIGN_ASSETS_PER_CAMPAIGN = 143
        PROMOTION_CUSTOMER_ASSETS_PER_CUSTOMER = 79
        PROMOTION_CAMPAIGN_ASSETS_PER_CAMPAIGN = 80
        PROMOTION_AD_GROUP_ASSETS_PER_AD_GROUP = 81
        CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER = 134
        CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN = 135
        CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP = 136
        SITELINK_CUSTOMER_ASSETS_PER_CUSTOMER = 137
        SITELINK_CAMPAIGN_ASSETS_PER_CAMPAIGN = 138
        SITELINK_AD_GROUP_ASSETS_PER_AD_GROUP = 139
        STRUCTURED_SNIPPET_CUSTOMER_ASSETS_PER_CUSTOMER = 140
        STRUCTURED_SNIPPET_CAMPAIGN_ASSETS_PER_CAMPAIGN = 141
        STRUCTURED_SNIPPET_AD_GROUP_ASSETS_PER_AD_GROUP = 142
        MOBILE_APP_CUSTOMER_ASSETS_PER_CUSTOMER = 144
        MOBILE_APP_CAMPAIGN_ASSETS_PER_CAMPAIGN = 145
        MOBILE_APP_AD_GROUP_ASSETS_PER_AD_GROUP = 146
        HOTEL_CALLOUT_CUSTOMER_ASSETS_PER_CUSTOMER = 147
        HOTEL_CALLOUT_CAMPAIGN_ASSETS_PER_CAMPAIGN = 148
        HOTEL_CALLOUT_AD_GROUP_ASSETS_PER_AD_GROUP = 149
        CALL_CUSTOMER_ASSETS_PER_CUSTOMER = 150
        CALL_CAMPAIGN_ASSETS_PER_CAMPAIGN = 151
        CALL_AD_GROUP_ASSETS_PER_AD_GROUP = 152
        PRICE_CUSTOMER_ASSETS_PER_CUSTOMER = 154
        PRICE_CAMPAIGN_ASSETS_PER_CAMPAIGN = 155
        PRICE_AD_GROUP_ASSETS_PER_AD_GROUP = 156
        AD_IMAGE_CAMPAIGN_ASSETS_PER_CAMPAIGN = 175
        AD_IMAGE_AD_GROUP_ASSETS_PER_AD_GROUP = 176
        PAGE_FEED_ASSET_SETS_PER_CUSTOMER = 157
        DYNAMIC_EDUCATION_FEED_ASSET_SETS_PER_CUSTOMER = 158
        ASSETS_PER_PAGE_FEED_ASSET_SET = 159
        ASSETS_PER_DYNAMIC_EDUCATION_FEED_ASSET_SET = 160
        DYNAMIC_REAL_ESTATE_ASSET_SETS_PER_CUSTOMER = 161
        ASSETS_PER_DYNAMIC_REAL_ESTATE_ASSET_SET = 162
        DYNAMIC_CUSTOM_ASSET_SETS_PER_CUSTOMER = 163
        ASSETS_PER_DYNAMIC_CUSTOM_ASSET_SET = 164
        DYNAMIC_HOTELS_AND_RENTALS_ASSET_SETS_PER_CUSTOMER = 165
        ASSETS_PER_DYNAMIC_HOTELS_AND_RENTALS_ASSET_SET = 166
        DYNAMIC_LOCAL_ASSET_SETS_PER_CUSTOMER = 167
        ASSETS_PER_DYNAMIC_LOCAL_ASSET_SET = 168
        DYNAMIC_FLIGHTS_ASSET_SETS_PER_CUSTOMER = 169
        ASSETS_PER_DYNAMIC_FLIGHTS_ASSET_SET = 170
        DYNAMIC_TRAVEL_ASSET_SETS_PER_CUSTOMER = 171
        ASSETS_PER_DYNAMIC_TRAVEL_ASSET_SET = 172
        DYNAMIC_JOBS_ASSET_SETS_PER_CUSTOMER = 173
        ASSETS_PER_DYNAMIC_JOBS_ASSET_SET = 174
        BUSINESS_NAME_CAMPAIGN_ASSETS_PER_CAMPAIGN = 179
        BUSINESS_LOGO_CAMPAIGN_ASSETS_PER_CAMPAIGN = 180
        VERSIONS_PER_AD = 82
        USER_FEEDS_PER_CUSTOMER = 90
        SYSTEM_FEEDS_PER_CUSTOMER = 91
        FEED_ATTRIBUTES_PER_FEED = 92
        FEED_ITEMS_PER_CUSTOMER = 94
        CAMPAIGN_FEEDS_PER_CUSTOMER = 95
        BASE_CAMPAIGN_FEEDS_PER_CUSTOMER = 96
        EXPERIMENT_CAMPAIGN_FEEDS_PER_CUSTOMER = 109
        AD_GROUP_FEEDS_PER_CUSTOMER = 97
        BASE_AD_GROUP_FEEDS_PER_CUSTOMER = 98
        EXPERIMENT_AD_GROUP_FEEDS_PER_CUSTOMER = 110
        AD_GROUP_FEEDS_PER_CAMPAIGN = 99
        FEED_ITEM_SETS_PER_CUSTOMER = 100
        FEED_ITEMS_PER_FEED_ITEM_SET = 101
        CAMPAIGN_EXPERIMENTS_PER_CUSTOMER = 112
        EXPERIMENT_ARMS_PER_VIDEO_EXPERIMENT = 113
        OWNED_LABELS_PER_CUSTOMER = 115
        LABELS_PER_CAMPAIGN = 117
        LABELS_PER_AD_GROUP = 118
        LABELS_PER_AD_GROUP_AD = 119
        LABELS_PER_AD_GROUP_CRITERION = 120
        TARGET_CUSTOMERS_PER_LABEL = 121
        KEYWORD_PLANS_PER_USER_PER_CUSTOMER = 122
        KEYWORD_PLAN_AD_GROUP_KEYWORDS_PER_KEYWORD_PLAN = 123
        KEYWORD_PLAN_AD_GROUPS_PER_KEYWORD_PLAN = 124
        KEYWORD_PLAN_NEGATIVE_KEYWORDS_PER_KEYWORD_PLAN = 125
        KEYWORD_PLAN_CAMPAIGNS_PER_KEYWORD_PLAN = 126
        CONVERSION_ACTIONS_PER_CUSTOMER = 128
        BATCH_JOB_OPERATIONS_PER_JOB = 130
        BATCH_JOBS_PER_CUSTOMER = 131
        HOTEL_CHECK_IN_DATE_RANGE_BID_MODIFIERS_PER_AD_GROUP = 132
        SHARED_SETS_PER_ACCOUNT_FOR_ACCOUNT_LEVEL_NEGATIVE_KEYWORDS = 177
        ACCOUNT_LEVEL_NEGATIVE_KEYWORDS_PER_SHARED_SET = 178
        ENABLED_ASSET_PER_HOTEL_PROPERTY_ASSET_SET = 181
        ENABLED_HOTEL_PROPERTY_ASSET_LINKS_PER_ASSET_GROUP = 182
        BRANDS_PER_SHARED_SET = 183
        ENABLED_BRAND_LIST_CRITERIA_PER_CAMPAIGN = 184
        SHARED_SETS_PER_ACCOUNT_FOR_BRAND = 185
        LOOKALIKE_USER_LISTS_PER_CUSTOMER = 186
        LOGO_CAMPAIGN_ASSETS_PER_CAMPAIGN = 187
        BUSINESS_MESSAGE_ASSET_LINKS_PER_CUSTOMER = 188
        WHATSAPP_BUSINESS_MESSAGE_ASSET_LINKS_PER_CAMPAIGN = 189
        WHATSAPP_BUSINESS_MESSAGE_ASSET_LINKS_PER_AD_GROUP = 190
        BRAND_LIST_CRITERIA_PER_AD_GROUP = 193


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v19.resources.types import (
    campaign_draft as gagr_campaign_draft,
)
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.services",
    marshal="google.ads.googleads.v19",
    manifest={
        "MutateCampaignDraftsRequest",
        "PromoteCampaignDraftRequest",
        "CampaignDraftOperation",
        "MutateCampaignDraftsResponse",
        "MutateCampaignDraftResult",
        "ListCampaignDraftAsyncErrorsRequest",
        "ListCampaignDraftAsyncErrorsResponse",
    },
)


class MutateCampaignDraftsRequest(proto.Message):
    r"""Request message for
    [CampaignDraftService.MutateCampaignDrafts][google.ads.googleads.v19.services.CampaignDraftService.MutateCampaignDrafts].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            campaign drafts are being modified.
        operations (MutableSequence[google.ads.googleads.v19.services.types.CampaignDraftOperation]):
            Required. The list of operations to perform
            on individual campaign drafts.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        response_content_type (google.ads.googleads.v19.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned post mutation.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CampaignDraftOperation"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="CampaignDraftOperation",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class PromoteCampaignDraftRequest(proto.Message):
    r"""Request message for
    [CampaignDraftService.PromoteCampaignDraft][google.ads.googleads.v19.services.CampaignDraftService.PromoteCampaignDraft].

    Attributes:
        campaign_draft (str):
            Required. The resource name of the campaign
            draft to promote.
        validate_only (bool):
            If true, the request is validated but no Long
            Running Operation is created. Only errors are
            returned.
    """

    campaign_draft: str = proto.Field(
        proto.STRING,
        number=1,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class CampaignDraftOperation(proto.Message):
    r"""A single operation (create, update, remove) on a campaign
    draft.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which resource
            fields are modified in an update.
        create (google.ads.googleads.v19.resources.types.CampaignDraft):
            Create operation: No resource name is
            expected for the new campaign draft.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v19.resources.types.CampaignDraft):
            Update operation: The campaign draft is
            expected to have a valid resource name.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: The campaign draft is expected to have a
            valid resource name, in this format:

            ``customers/{customer_id}/campaignDrafts/{base_campaign_id}~{draft_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: gagr_campaign_draft.CampaignDraft = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=gagr_campaign_draft.CampaignDraft,
    )
    update: gagr_campaign_draft.CampaignDraft = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="operation",
        message=gagr_campaign_draft.CampaignDraft,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=3,
        oneof="operation",
    )


class MutateCampaignDraftsResponse(proto.Message):
    r"""Response message for campaign draft mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v19.services.types.MutateCampaignDraftResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateCampaignDraftResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="MutateCampaignDraftResult",
    )


class MutateCampaignDraftResult(proto.Message):
    r"""The result for the campaign draft mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
        campaign_draft (google.ads.googleads.v19.resources.types.CampaignDraft):
            The mutated campaign draft with only mutable fields after
            mutate. The field will only be returned when
            response_content_type is set to "MUTABLE_RESOURCE".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign_draft: gagr_campaign_draft.CampaignDraft = proto.Field(
        proto.MESSAGE,
        number=2,
        message=gagr_campaign_draft.CampaignDraft,
    )


class ListCampaignDraftAsyncErrorsRequest(proto.Message):
    r"""Request message for
    [CampaignDraftService.ListCampaignDraftAsyncErrors][google.ads.googleads.v19.services.CampaignDraftService.ListCampaignDraftAsyncErrors].

    Attributes:
        resource_name (str):
            Required. The name of the campaign draft from
            which to retrieve the async errors.
        page_token (str):
            Token of the page to retrieve. If not specified, the first
            page of results will be returned. Use the value obtained
            from ``next_page_token`` in the previous response in order
            to request the next page of results.
        page_size (int):
            Number of elements to retrieve in a single
            page. When a page request is too large, the
            server may decide to further limit the number of
            returned resources.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )


class ListCampaignDraftAsyncErrorsResponse(proto.Message):
    r"""Response message for
    [CampaignDraftService.ListCampaignDraftAsyncErrors][google.ads.googleads.v19.services.CampaignDraftService.ListCampaignDraftAsyncErrors].

    Attributes:
        errors (MutableSequence[google.rpc.status_pb2.Status]):
            Details of the errors when performing the
            asynchronous operation.
        next_page_token (str):
            Pagination token used to retrieve the next page of results.
            Pass the content of this string as the ``page_token``
            attribute of the next request. ``next_page_token`` is not
            returned for the last page.
    """

    @property
    def raw_page(self):
        return self

    errors: MutableSequence[status_pb2.Status] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "HotelReconciliationStatusEnum",
    },
)


class HotelReconciliationStatusEnum(proto.Message):
    r"""Container for HotelReconciliationStatus."""

    class HotelReconciliationStatus(proto.Enum):
        r"""Status of the hotel booking reconciliation.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            RESERVATION_ENABLED (2):
                Bookings are for a future date, or a stay is
                underway but the check-out date hasn't passed.
                An active reservation can't be reconciled.
            RECONCILIATION_NEEDED (3):
                Check-out has already taken place, or the
                booked dates have passed without cancellation.
                Bookings that are not reconciled within 45 days
                of the check-out date are billed based on the
                original booking price.
            RECONCILED (4):
                These bookings have been reconciled.
                Reconciled bookings are billed 45 days after the
                check-out date.
            CANCELED (5):
                This booking was marked as canceled. Canceled
                stays with a value greater than zero (due to
                minimum stay rules or cancellation fees) are
                billed 45 days after the check-out date.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        RESERVATION_ENABLED = 2
        RECONCILIATION_NEEDED = 3
        RECONCILED = 4
        CANCELED = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import data_link_status
from google.ads.googleads.v20.enums.types import data_link_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "DataLink",
        "YoutubeVideoIdentifier",
    },
)


class DataLink(proto.Message):
    r"""Represents the data sharing connection between  a Google
    Ads customer and another product's data.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. Resource name of the product data link. DataLink
            resource names have the form:

            \`customers/{customer_id}/datalinks/{product_link_id}~{data_link_id}}
        product_link_id (int):
            Output only. The ID of the link.
            This field is read only.

            This field is a member of `oneof`_ ``_product_link_id``.
        data_link_id (int):
            Output only. The ID of the data link.
            This field is read only.

            This field is a member of `oneof`_ ``_data_link_id``.
        type_ (google.ads.googleads.v20.enums.types.DataLinkTypeEnum.DataLinkType):
            Output only. The type of the data.
        status (google.ads.googleads.v20.enums.types.DataLinkStatusEnum.DataLinkStatus):
            Output only. The status of the data link.
        youtube_video (google.ads.googleads.v20.resources.types.YoutubeVideoIdentifier):
            Immutable. A data link to YouTube video.

            This field is a member of `oneof`_ ``data_link_entity``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    product_link_id: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    data_link_id: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    type_: data_link_type.DataLinkTypeEnum.DataLinkType = proto.Field(
        proto.ENUM,
        number=4,
        enum=data_link_type.DataLinkTypeEnum.DataLinkType,
    )
    status: data_link_status.DataLinkStatusEnum.DataLinkStatus = proto.Field(
        proto.ENUM,
        number=5,
        enum=data_link_status.DataLinkStatusEnum.DataLinkStatus,
    )
    youtube_video: "YoutubeVideoIdentifier" = proto.Field(
        proto.MESSAGE,
        number=6,
        oneof="data_link_entity",
        message="YoutubeVideoIdentifier",
    )


class YoutubeVideoIdentifier(proto.Message):
    r"""The identifier for YouTube video

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        channel_id (str):
            Immutable. The ID of the hosting channel of
            the video. This is a string value with “UC”
            prefix. For example, "UCK8sQmJBp8GCxrOtXWBpyEA".

            This field is a member of `oneof`_ ``_channel_id``.
        video_id (str):
            Immutable. The ID of the video associated
            with the video link. This is the 11 character
            string value used in the YouTube video URL. For
            example, video ID is jV1vkHv4zq8 from the
            YouTube video URL
            "https://www.youtube.com/watch?v=jV1vkHv4zq8&t=2s".

            This field is a member of `oneof`_ ``_video_id``.
    """

    channel_id: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    video_id: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.ads.googleads.v20 import gapic_version as package_version

__version__ = package_version.__version__


from .types.ad_asset import AdAppDeepLinkAsset
from .types.ad_asset import AdCallToActionAsset
from .types.ad_asset import AdDemandGenCarouselCardAsset
from .types.ad_asset import AdImageAsset
from .types.ad_asset import AdMediaBundleAsset
from .types.ad_asset import AdTextAsset
from .types.ad_asset import AdVideoAsset
from .types.ad_asset import AdVideoAssetInfo
from .types.ad_asset import AdVideoAssetInventoryPreferences
from .types.ad_type_infos import AppAdInfo
from .types.ad_type_infos import AppEngagementAdInfo
from .types.ad_type_infos import AppPreRegistrationAdInfo
from .types.ad_type_infos import CallAdInfo
from .types.ad_type_infos import DemandGenCarouselAdInfo
from .types.ad_type_infos import DemandGenMultiAssetAdInfo
from .types.ad_type_infos import DemandGenProductAdInfo
from .types.ad_type_infos import DemandGenVideoResponsiveAdInfo
from .types.ad_type_infos import DisplayUploadAdInfo
from .types.ad_type_infos import ExpandedDynamicSearchAdInfo
from .types.ad_type_infos import ExpandedTextAdInfo
from .types.ad_type_infos import HotelAdInfo
from .types.ad_type_infos import ImageAdInfo
from .types.ad_type_infos import InFeedVideoAdInfo
from .types.ad_type_infos import LegacyAppInstallAdInfo
from .types.ad_type_infos import LegacyResponsiveDisplayAdInfo
from .types.ad_type_infos import LocalAdInfo
from .types.ad_type_infos import ResponsiveDisplayAdControlSpec
from .types.ad_type_infos import ResponsiveDisplayAdInfo
from .types.ad_type_infos import ResponsiveSearchAdInfo
from .types.ad_type_infos import ShoppingComparisonListingAdInfo
from .types.ad_type_infos import ShoppingProductAdInfo
from .types.ad_type_infos import ShoppingSmartAdInfo
from .types.ad_type_infos import SmartCampaignAdInfo
from .types.ad_type_infos import TextAdInfo
from .types.ad_type_infos import TravelAdInfo
from .types.ad_type_infos import VideoAdInfo
from .types.ad_type_infos import VideoBumperInStreamAdInfo
from .types.ad_type_infos import VideoNonSkippableInStreamAdInfo
from .types.ad_type_infos import VideoOutstreamAdInfo
from .types.ad_type_infos import VideoResponsiveAdInfo
from .types.ad_type_infos import VideoTrueViewInStreamAdInfo
from .types.ad_type_infos import YouTubeAudioAdInfo
from .types.additional_application_info import AdditionalApplicationInfo
from .types.asset_policy import AdAssetPolicySummary
from .types.asset_policy import AssetDisapproved
from .types.asset_policy import AssetLinkPrimaryStatusDetails
from .types.asset_set_types import BusinessProfileBusinessNameFilter
from .types.asset_set_types import BusinessProfileLocationGroup
from .types.asset_set_types import BusinessProfileLocationSet
from .types.asset_set_types import ChainFilter
from .types.asset_set_types import ChainLocationGroup
from .types.asset_set_types import ChainSet
from .types.asset_set_types import DynamicBusinessProfileLocationGroupFilter
from .types.asset_set_types import LocationSet
from .types.asset_set_types import MapsLocationInfo
from .types.asset_set_types import MapsLocationSet
from .types.asset_types import AppDeepLinkAsset
from .types.asset_types import BookOnGoogleAsset
from .types.asset_types import BusinessMessageAsset
from .types.asset_types import BusinessMessageCallToActionInfo
from .types.asset_types import BusinessProfileLocation
from .types.asset_types import CallAsset
from .types.asset_types import CalloutAsset
from .types.asset_types import CallToActionAsset
from .types.asset_types import DemandGenCarouselCardAsset
from .types.asset_types import DynamicCustomAsset
from .types.asset_types import DynamicEducationAsset
from .types.asset_types import DynamicFlightsAsset
from .types.asset_types import DynamicHotelsAndRentalsAsset
from .types.asset_types import DynamicJobsAsset
from .types.asset_types import DynamicLocalAsset
from .types.asset_types import DynamicRealEstateAsset
from .types.asset_types import DynamicTravelAsset
from .types.asset_types import HotelCalloutAsset
from .types.asset_types import HotelPropertyAsset
from .types.asset_types import ImageAsset
from .types.asset_types import ImageDimension
from .types.asset_types import LeadFormAsset
from .types.asset_types import LeadFormCustomQuestionField
from .types.asset_types import LeadFormDeliveryMethod
from .types.asset_types import LeadFormField
from .types.asset_types import LeadFormSingleChoiceAnswers
from .types.asset_types import LocationAsset
from .types.asset_types import MediaBundleAsset
from .types.asset_types import MobileAppAsset
from .types.asset_types import PageFeedAsset
from .types.asset_types import PriceAsset
from .types.asset_types import PriceOffering
from .types.asset_types import PromotionAsset
from .types.asset_types import SitelinkAsset
from .types.asset_types import StructuredSnippetAsset
from .types.asset_types import TextAsset
from .types.asset_types import WebhookDelivery
from .types.asset_types import WhatsappBusinessMessageInfo
from .types.asset_types import YoutubeVideoAsset
from .types.asset_usage import AssetUsage
from .types.audience_insights_attribute import AudienceInsightsAttribute
from .types.audience_insights_attribute import AudienceInsightsAttributeMetadata
from .types.audience_insights_attribute import AudienceInsightsCategory
from .types.audience_insights_attribute import AudienceInsightsEntity
from .types.audience_insights_attribute import AudienceInsightsLineup
from .types.audience_insights_attribute import AudienceInsightsTopic
from .types.audience_insights_attribute import KnowledgeGraphAttributeMetadata
from .types.audience_insights_attribute import LineupAttributeMetadata
from .types.audience_insights_attribute import LocationAttributeMetadata
from .types.audience_insights_attribute import UserInterestAttributeMetadata
from .types.audience_insights_attribute import YouTubeChannelAttributeMetadata
from .types.audience_insights_attribute import YouTubeVideoAttributeMetadata
from .types.audiences import AgeDimension
from .types.audiences import AgeSegment
from .types.audiences import AudienceDimension
from .types.audiences import AudienceExclusionDimension
from .types.audiences import AudienceSegment
from .types.audiences import AudienceSegmentDimension
from .types.audiences import CustomAudienceSegment
from .types.audiences import DetailedDemographicSegment
from .types.audiences import ExclusionSegment
from .types.audiences import GenderDimension
from .types.audiences import HouseholdIncomeDimension
from .types.audiences import LifeEventSegment
from .types.audiences import ParentalStatusDimension
from .types.audiences import UserInterestSegment
from .types.audiences import UserListSegment
from .types.bidding import Commission
from .types.bidding import EnhancedCpc
from .types.bidding import FixedCpm
from .types.bidding import FixedCpmTargetFrequencyGoalInfo
from .types.bidding import ManualCpa
from .types.bidding import ManualCpc
from .types.bidding import ManualCpm
from .types.bidding import ManualCpv
from .types.bidding import MaximizeConversions
from .types.bidding import MaximizeConversionValue
from .types.bidding import PercentCpc
from .types.bidding import TargetCpa
from .types.bidding import TargetCpm
from .types.bidding import TargetCpmTargetFrequencyGoal
from .types.bidding import TargetCpv
from .types.bidding import TargetImpressionShare
from .types.bidding import TargetRoas
from .types.bidding import TargetSpend
from .types.click_location import ClickLocation
from .types.consent import Consent
from .types.criteria import ActivityCityInfo
from .types.criteria import ActivityCountryInfo
from .types.criteria import ActivityIdInfo
from .types.criteria import ActivityRatingInfo
from .types.criteria import ActivityStateInfo
from .types.criteria import AddressInfo
from .types.criteria import AdScheduleInfo
from .types.criteria import AgeRangeInfo
from .types.criteria import AppPaymentModelInfo
from .types.criteria import AudienceInfo
from .types.criteria import BrandInfo
from .types.criteria import BrandListInfo
from .types.criteria import CarrierInfo
from .types.criteria import CombinedAudienceInfo
from .types.criteria import ContentLabelInfo
from .types.criteria import CustomAffinityInfo
from .types.criteria import CustomAudienceInfo
from .types.criteria import CustomIntentInfo
from .types.criteria import DeviceInfo
from .types.criteria import GenderInfo
from .types.criteria import GeoPointInfo
from .types.criteria import HotelAdvanceBookingWindowInfo
from .types.criteria import HotelCheckInDateRangeInfo
from .types.criteria import HotelCheckInDayInfo
from .types.criteria import HotelCityInfo
from .types.criteria import HotelClassInfo
from .types.criteria import HotelCountryRegionInfo
from .types.criteria import HotelDateSelectionTypeInfo
from .types.criteria import HotelIdInfo
from .types.criteria import HotelLengthOfStayInfo
from .types.criteria import HotelStateInfo
from .types.criteria import IncomeRangeInfo
from .types.criteria import InteractionTypeInfo
from .types.criteria import IpBlockInfo
from .types.criteria import KeywordInfo
from .types.criteria import KeywordThemeInfo
from .types.criteria import LanguageInfo
from .types.criteria import ListingDimensionInfo
from .types.criteria import ListingDimensionPath
from .types.criteria import ListingGroupInfo
from .types.criteria import ListingScopeInfo
from .types.criteria import LocalServiceIdInfo
from .types.criteria import LocationGroupInfo
from .types.criteria import LocationInfo
from .types.criteria import MobileAppCategoryInfo
from .types.criteria import MobileApplicationInfo
from .types.criteria import MobileDeviceInfo
from .types.criteria import NegativeKeywordListInfo
from .types.criteria import OperatingSystemVersionInfo
from .types.criteria import ParentalStatusInfo
from .types.criteria import PlacementInfo
from .types.criteria import ProductBrandInfo
from .types.criteria import ProductCategoryInfo
from .types.criteria import ProductChannelExclusivityInfo
from .types.criteria import ProductChannelInfo
from .types.criteria import ProductConditionInfo
from .types.criteria import ProductCustomAttributeInfo
from .types.criteria import ProductGroupingInfo
from .types.criteria import ProductItemIdInfo
from .types.criteria import ProductLabelsInfo
from .types.criteria import ProductLegacyConditionInfo
from .types.criteria import ProductTypeFullInfo
from .types.criteria import ProductTypeInfo
from .types.criteria import ProximityInfo
from .types.criteria import SearchThemeInfo
from .types.criteria import TopicInfo
from .types.criteria import UnknownListingDimensionInfo
from .types.criteria import UserInterestInfo
from .types.criteria import UserListInfo
from .types.criteria import WebpageConditionInfo
from .types.criteria import WebpageInfo
from .types.criteria import WebpageListInfo
from .types.criteria import WebpageSampleInfo
from .types.criteria import YouTubeChannelInfo
from .types.criteria import YouTubeVideoInfo
from .types.criterion_category_availability import CriterionCategoryAvailability
from .types.criterion_category_availability import (
    CriterionCategoryChannelAvailability,
)
from .types.criterion_category_availability import (
    CriterionCategoryLocaleAvailability,
)
from .types.custom_parameter import CustomParameter
from .types.customizer_value import CustomizerValue
from .types.dates import DateRange
from .types.dates import YearMonth
from .types.dates import YearMonthRange
from .types.extensions import CallFeedItem
from .types.extensions import CalloutFeedItem
from .types.extensions import SitelinkFeedItem
from .types.feed_common import Money
from .types.final_app_url import FinalAppUrl
from .types.frequency_cap import FrequencyCapEntry
from .types.frequency_cap import FrequencyCapKey
from .types.keyword_plan_common import ConceptGroup
from .types.keyword_plan_common import HistoricalMetricsOptions
from .types.keyword_plan_common import KeywordAnnotations
from .types.keyword_plan_common import KeywordConcept
from .types.keyword_plan_common import KeywordPlanAggregateMetricResults
from .types.keyword_plan_common import KeywordPlanAggregateMetrics
from .types.keyword_plan_common import KeywordPlanDeviceSearches
from .types.keyword_plan_common import KeywordPlanHistoricalMetrics
from .types.keyword_plan_common import MonthlySearchVolume
from .types.lifecycle_goals import LifecycleGoalValueSettings
from .types.local_services import LocalServicesDocumentReadOnly
from .types.metric_goal import MetricGoal
from .types.metrics import Metrics
from .types.metrics import SearchVolumeRange
from .types.offline_user_data import CustomerMatchUserListMetadata
from .types.offline_user_data import EventAttribute
from .types.offline_user_data import EventItemAttribute
from .types.offline_user_data import ItemAttribute
from .types.offline_user_data import OfflineUserAddressInfo
from .types.offline_user_data import ShoppingLoyalty
from .types.offline_user_data import StoreAttribute
from .types.offline_user_data import StoreSalesMetadata
from .types.offline_user_data import StoreSalesThirdPartyMetadata
from .types.offline_user_data import TransactionAttribute
from .types.offline_user_data import UserAttribute
from .types.offline_user_data import UserData
from .types.offline_user_data import UserIdentifier
from .types.policy import PolicyTopicConstraint
from .types.policy import PolicyTopicEntry
from .types.policy import PolicyTopicEvidence
from .types.policy import PolicyValidationParameter
from .types.policy import PolicyViolationKey
from .types.policy_summary import PolicySummary
from .types.real_time_bidding_setting import RealTimeBiddingSetting
from .types.segments import AssetInteractionTarget
from .types.segments import BudgetCampaignAssociationStatus
from .types.segments import Keyword
from .types.segments import Segments
from .types.segments import SkAdNetworkSourceApp
from .types.simulation import BudgetSimulationPoint
from .types.simulation import BudgetSimulationPointList
from .types.simulation import CpcBidSimulationPoint
from .types.simulation import CpcBidSimulationPointList
from .types.simulation import CpvBidSimulationPoint
from .types.simulation import CpvBidSimulationPointList
from .types.simulation import PercentCpcBidSimulationPoint
from .types.simulation import PercentCpcBidSimulationPointList
from .types.simulation import TargetCpaSimulationPoint
from .types.simulation import TargetCpaSimulationPointList
from .types.simulation import TargetImpressionShareSimulationPoint
from .types.simulation import TargetImpressionShareSimulationPointList
from .types.simulation import TargetRoasSimulationPoint
from .types.simulation import TargetRoasSimulationPointList
from .types.tag_snippet import TagSnippet
from .types.targeting_setting import TargetingSetting
from .types.targeting_setting import TargetRestriction
from .types.targeting_setting import TargetRestrictionOperation
from .types.text_label import TextLabel
from .types.url_collection import UrlCollection
from .types.user_lists import BasicUserListInfo
from .types.user_lists import CrmBasedUserListInfo
from .types.user_lists import FlexibleRuleOperandInfo
from .types.user_lists import FlexibleRuleUserListInfo
from .types.user_lists import LogicalUserListInfo
from .types.user_lists import LogicalUserListOperandInfo
from .types.user_lists import LookalikeUserListInfo
from .types.user_lists import RuleBasedUserListInfo
from .types.user_lists import SimilarUserListInfo
from .types.user_lists import UserListActionInfo
from .types.user_lists import UserListDateRuleItemInfo
from .types.user_lists import UserListLogicalRuleInfo
from .types.user_lists import UserListNumberRuleItemInfo
from .types.user_lists import UserListRuleInfo
from .types.user_lists import UserListRuleItemGroupInfo
from .types.user_lists import UserListRuleItemInfo
from .types.user_lists import UserListStringRuleItemInfo
from .types.value import Value

__all__ = (
    "ActivityCityInfo",
    "ActivityCountryInfo",
    "ActivityIdInfo",
    "ActivityRatingInfo",
    "ActivityStateInfo",
    "AdAppDeepLinkAsset",
    "AdAssetPolicySummary",
    "AdCallToActionAsset",
    "AdDemandGenCarouselCardAsset",
    "AdImageAsset",
    "AdMediaBundleAsset",
    "AdScheduleInfo",
    "AdTextAsset",
    "AdVideoAsset",
    "AdVideoAssetInfo",
    "AdVideoAssetInventoryPreferences",
    "AdditionalApplicationInfo",
    "AddressInfo",
    "AgeDimension",
    "AgeRangeInfo",
    "AgeSegment",
    "AppAdInfo",
    "AppDeepLinkAsset",
    "AppEngagementAdInfo",
    "AppPaymentModelInfo",
    "AppPreRegistrationAdInfo",
    "AssetDisapproved",
    "AssetInteractionTarget",
    "AssetLinkPrimaryStatusDetails",
    "AssetUsage",
    "AudienceDimension",
    "AudienceExclusionDimension",
    "AudienceInfo",
    "AudienceInsightsAttribute",
    "AudienceInsightsAttributeMetadata",
    "AudienceInsightsCategory",
    "AudienceInsightsEntity",
    "AudienceInsightsLineup",
    "AudienceInsightsTopic",
    "AudienceSegment",
    "AudienceSegmentDimension",
    "BasicUserListInfo",
    "BookOnGoogleAsset",
    "BrandInfo",
    "BrandListInfo",
    "BudgetCampaignAssociationStatus",
    "BudgetSimulationPoint",
    "BudgetSimulationPointList",
    "BusinessMessageAsset",
    "BusinessMessageCallToActionInfo",
    "BusinessProfileBusinessNameFilter",
    "BusinessProfileLocation",
    "BusinessProfileLocationGroup",
    "BusinessProfileLocationSet",
    "CallAdInfo",
    "CallAsset",
    "CallFeedItem",
    "CallToActionAsset",
    "CalloutAsset",
    "CalloutFeedItem",
    "CarrierInfo",
    "ChainFilter",
    "ChainLocationGroup",
    "ChainSet",
    "ClickLocation",
    "CombinedAudienceInfo",
    "Commission",
    "ConceptGroup",
    "Consent",
    "ContentLabelInfo",
    "CpcBidSimulationPoint",
    "CpcBidSimulationPointList",
    "CpvBidSimulationPoint",
    "CpvBidSimulationPointList",
    "CriterionCategoryAvailability",
    "CriterionCategoryChannelAvailability",
    "CriterionCategoryLocaleAvailability",
    "CrmBasedUserListInfo",
    "CustomAffinityInfo",
    "CustomAudienceInfo",
    "CustomAudienceSegment",
    "CustomIntentInfo",
    "CustomParameter",
    "CustomerMatchUserListMetadata",
    "CustomizerValue",
    "DateRange",
    "DemandGenCarouselAdInfo",
    "DemandGenCarouselCardAsset",
    "DemandGenMultiAssetAdInfo",
    "DemandGenProductAdInfo",
    "DemandGenVideoResponsiveAdInfo",
    "DetailedDemographicSegment",
    "DeviceInfo",
    "DisplayUploadAdInfo",
    "DynamicBusinessProfileLocationGroupFilter",
    "DynamicCustomAsset",
    "DynamicEducationAsset",
    "DynamicFlightsAsset",
    "DynamicHotelsAndRentalsAsset",
    "DynamicJobsAsset",
    "DynamicLocalAsset",
    "DynamicRealEstateAsset",
    "DynamicTravelAsset",
    "EnhancedCpc",
    "EventAttribute",
    "EventItemAttribute",
    "ExclusionSegment",
    "ExpandedDynamicSearchAdInfo",
    "ExpandedTextAdInfo",
    "FinalAppUrl",
    "FixedCpm",
    "FixedCpmTargetFrequencyGoalInfo",
    "FlexibleRuleOperandInfo",
    "FlexibleRuleUserListInfo",
    "FrequencyCapEntry",
    "FrequencyCapKey",
    "GenderDimension",
    "GenderInfo",
    "GeoPointInfo",
    "HistoricalMetricsOptions",
    "HotelAdInfo",
    "HotelAdvanceBookingWindowInfo",
    "HotelCalloutAsset",
    "HotelCheckInDateRangeInfo",
    "HotelCheckInDayInfo",
    "HotelCityInfo",
    "HotelClassInfo",
    "HotelCountryRegionInfo",
    "HotelDateSelectionTypeInfo",
    "HotelIdInfo",
    "HotelLengthOfStayInfo",
    "HotelPropertyAsset",
    "HotelStateInfo",
    "HouseholdIncomeDimension",
    "ImageAdInfo",
    "ImageAsset",
    "ImageDimension",
    "InFeedVideoAdInfo",
    "IncomeRangeInfo",
    "InteractionTypeInfo",
    "IpBlockInfo",
    "ItemAttribute",
    "Keyword",
    "KeywordAnnotations",
    "KeywordConcept",
    "KeywordInfo",
    "KeywordPlanAggregateMetricResults",
    "KeywordPlanAggregateMetrics",
    "KeywordPlanDeviceSearches",
    "KeywordPlanHistoricalMetrics",
    "KeywordThemeInfo",
    "KnowledgeGraphAttributeMetadata",
    "LanguageInfo",
    "LeadFormAsset",
    "LeadFormCustomQuestionField",
    "LeadFormDeliveryMethod",
    "LeadFormField",
    "LeadFormSingleChoiceAnswers",
    "LegacyAppInstallAdInfo",
    "LegacyResponsiveDisplayAdInfo",
    "LifeEventSegment",
    "LifecycleGoalValueSettings",
    "LineupAttributeMetadata",
    "ListingDimensionInfo",
    "ListingDimensionPath",
    "ListingGroupInfo",
    "ListingScopeInfo",
    "LocalAdInfo",
    "LocalServiceIdInfo",
    "LocalServicesDocumentReadOnly",
    "LocationAsset",
    "LocationAttributeMetadata",
    "LocationGroupInfo",
    "LocationInfo",
    "LocationSet",
    "LogicalUserListInfo",
    "LogicalUserListOperandInfo",
    "LookalikeUserListInfo",
    "ManualCpa",
    "ManualCpc",
    "ManualCpm",
    "ManualCpv",
    "MapsLocationInfo",
    "MapsLocationSet",
    "MaximizeConversionValue",
    "MaximizeConversions",
    "MediaBundleAsset",
    "MetricGoal",
    "Metrics",
    "MobileAppAsset",
    "MobileAppCategoryInfo",
    "MobileApplicationInfo",
    "MobileDeviceInfo",
    "Money",
    "MonthlySearchVolume",
    "NegativeKeywordListInfo",
    "OfflineUserAddressInfo",
    "OperatingSystemVersionInfo",
    "PageFeedAsset",
    "ParentalStatusDimension",
    "ParentalStatusInfo",
    "PercentCpc",
    "PercentCpcBidSimulationPoint",
    "PercentCpcBidSimulationPointList",
    "PlacementInfo",
    "PolicySummary",
    "PolicyTopicConstraint",
    "PolicyTopicEntry",
    "PolicyTopicEvidence",
    "PolicyValidationParameter",
    "PolicyViolationKey",
    "PriceAsset",
    "PriceOffering",
    "ProductBrandInfo",
    "ProductCategoryInfo",
    "ProductChannelExclusivityInfo",
    "ProductChannelInfo",
    "ProductConditionInfo",
    "ProductCustomAttributeInfo",
    "ProductGroupingInfo",
    "ProductItemIdInfo",
    "ProductLabelsInfo",
    "ProductLegacyConditionInfo",
    "ProductTypeFullInfo",
    "ProductTypeInfo",
    "PromotionAsset",
    "ProximityInfo",
    "RealTimeBiddingSetting",
    "ResponsiveDisplayAdControlSpec",
    "ResponsiveDisplayAdInfo",
    "ResponsiveSearchAdInfo",
    "RuleBasedUserListInfo",
    "SearchThemeInfo",
    "SearchVolumeRange",
    "Segments",
    "ShoppingComparisonListingAdInfo",
    "ShoppingLoyalty",
    "ShoppingProductAdInfo",
    "ShoppingSmartAdInfo",
    "SimilarUserListInfo",
    "SitelinkAsset",
    "SitelinkFeedItem",
    "SkAdNetworkSourceApp",
    "SmartCampaignAdInfo",
    "StoreAttribute",
    "StoreSalesMetadata",
    "StoreSalesThirdPartyMetadata",
    "StructuredSnippetAsset",
    "TagSnippet",
    "TargetCpa",
    "TargetCpaSimulationPoint",
    "TargetCpaSimulationPointList",
    "TargetCpm",
    "TargetCpmTargetFrequencyGoal",
    "TargetCpv",
    "TargetImpressionShare",
    "TargetImpressionShareSimulationPoint",
    "TargetImpressionShareSimulationPointList",
    "TargetRestriction",
    "TargetRestrictionOperation",
    "TargetRoas",
    "TargetRoasSimulationPoint",
    "TargetRoasSimulationPointList",
    "TargetSpend",
    "TargetingSetting",
    "TextAdInfo",
    "TextAsset",
    "TextLabel",
    "TopicInfo",
    "TransactionAttribute",
    "TravelAdInfo",
    "UnknownListingDimensionInfo",
    "UrlCollection",
    "UserAttribute",
    "UserData",
    "UserIdentifier",
    "UserInterestAttributeMetadata",
    "UserInterestInfo",
    "UserInterestSegment",
    "UserListActionInfo",
    "UserListDateRuleItemInfo",
    "UserListInfo",
    "UserListLogicalRuleInfo",
    "UserListNumberRuleItemInfo",
    "UserListRuleInfo",
    "UserListRuleItemGroupInfo",
    "UserListRuleItemInfo",
    "UserListSegment",
    "UserListStringRuleItemInfo",
    "Value",
    "VideoAdInfo",
    "VideoBumperInStreamAdInfo",
    "VideoNonSkippableInStreamAdInfo",
    "VideoOutstreamAdInfo",
    "VideoResponsiveAdInfo",
    "VideoTrueViewInStreamAdInfo",
    "WebhookDelivery",
    "WebpageConditionInfo",
    "WebpageInfo",
    "WebpageListInfo",
    "WebpageSampleInfo",
    "WhatsappBusinessMessageInfo",
    "YearMonth",
    "YearMonthRange",
    "YouTubeAudioAdInfo",
    "YouTubeChannelAttributeMetadata",
    "YouTubeChannelInfo",
    "YouTubeVideoAttributeMetadata",
    "YouTubeVideoInfo",
    "YoutubeVideoAsset",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import (
    third_party_integration_partners as gagc_third_party_integration_partners,
)
from google.ads.googleads.v21.enums.types import brand_safety_suitability
from google.ads.googleads.v21.enums.types import conversion_tracking_status_enum
from google.ads.googleads.v21.enums.types import (
    customer_pay_per_conversion_eligibility_failure_reason,
)
from google.ads.googleads.v21.enums.types import customer_status
from google.ads.googleads.v21.enums.types import (
    local_services_verification_status,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.resources",
    marshal="google.ads.googleads.v21",
    manifest={
        "Customer",
        "CallReportingSetting",
        "ConversionTrackingSetting",
        "RemarketingSetting",
        "CustomerAgreementSetting",
        "LocalServicesSettings",
        "GranularLicenseStatus",
        "GranularInsuranceStatus",
        "VideoCustomer",
    },
)


class Customer(proto.Message):
    r"""A customer.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the customer. Customer
            resource names have the form:

            ``customers/{customer_id}``
        id (int):
            Output only. The ID of the customer.

            This field is a member of `oneof`_ ``_id``.
        descriptive_name (str):
            Optional, non-unique descriptive name of the
            customer.

            This field is a member of `oneof`_ ``_descriptive_name``.
        currency_code (str):
            Immutable. The currency in which the account
            operates. A subset of the currency codes from
            the ISO 4217 standard is supported.

            This field is a member of `oneof`_ ``_currency_code``.
        time_zone (str):
            Immutable. The local timezone ID of the
            customer.

            This field is a member of `oneof`_ ``_time_zone``.
        tracking_url_template (str):
            The URL template for constructing a tracking URL out of
            parameters. Only mutable in an ``update`` operation.

            This field is a member of `oneof`_ ``_tracking_url_template``.
        final_url_suffix (str):
            The URL template for appending params to the final URL. Only
            mutable in an ``update`` operation.

            This field is a member of `oneof`_ ``_final_url_suffix``.
        auto_tagging_enabled (bool):
            Whether auto-tagging is enabled for the customer. Only
            mutable in an ``update`` operation.

            This field is a member of `oneof`_ ``_auto_tagging_enabled``.
        has_partners_badge (bool):
            Output only. Whether the Customer has a
            Partners program badge. If the Customer is not
            associated with the Partners program, this will
            be false. For more information, see
            https://support.google.com/partners/answer/3125774.

            This field is a member of `oneof`_ ``_has_partners_badge``.
        manager (bool):
            Output only. Whether the customer is a
            manager.

            This field is a member of `oneof`_ ``_manager``.
        test_account (bool):
            Output only. Whether the customer is a test
            account.

            This field is a member of `oneof`_ ``_test_account``.
        call_reporting_setting (google.ads.googleads.v21.resources.types.CallReportingSetting):
            Call reporting setting for a customer. Only mutable in an
            ``update`` operation.
        conversion_tracking_setting (google.ads.googleads.v21.resources.types.ConversionTrackingSetting):
            Conversion tracking setting for a customer.
        remarketing_setting (google.ads.googleads.v21.resources.types.RemarketingSetting):
            Output only. Remarketing setting for a
            customer.
        pay_per_conversion_eligibility_failure_reasons (MutableSequence[google.ads.googleads.v21.enums.types.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason]):
            Output only. Reasons why the customer is not
            eligible to use PaymentMode.CONVERSIONS. If the
            list is empty, the customer is eligible. This
            field is read-only.
        optimization_score (float):
            Output only. Optimization score of the
            customer.
            Optimization score is an estimate of how well a
            customer's campaigns are set to perform. It
            ranges from 0% (0.0) to 100% (1.0). This field
            is null for all manager customers, and for
            unscored non-manager customers.

            See "About optimization score" at
            https://support.google.com/google-ads/answer/9061546.

            This field is read-only.

            This field is a member of `oneof`_ ``_optimization_score``.
        optimization_score_weight (float):
            Output only. Optimization score weight of the customer.

            Optimization score weight can be used to compare/aggregate
            optimization scores across multiple non-manager customers.
            The aggregate optimization score of a manager is computed as
            the sum over all of their customers of
            ``Customer.optimization_score * Customer.optimization_score_weight``.
            This field is 0 for all manager customers, and for unscored
            non-manager customers.

            This field is read-only.
        status (google.ads.googleads.v21.enums.types.CustomerStatusEnum.CustomerStatus):
            Output only. The status of the customer.
        location_asset_auto_migration_done (bool):
            Output only. True if feed based location has
            been migrated to asset based location.

            This field is a member of `oneof`_ ``_location_asset_auto_migration_done``.
        image_asset_auto_migration_done (bool):
            Output only. True if feed based image has
            been migrated to asset based image.

            This field is a member of `oneof`_ ``_image_asset_auto_migration_done``.
        location_asset_auto_migration_done_date_time (str):
            Output only. Timestamp of migration from feed
            based location to asset base location in
            yyyy-MM-dd HH:mm:ss format.

            This field is a member of `oneof`_ ``_location_asset_auto_migration_done_date_time``.
        image_asset_auto_migration_done_date_time (str):
            Output only. Timestamp of migration from feed
            based image to asset base image in yyyy-MM-dd
            HH:mm:ss format.

            This field is a member of `oneof`_ ``_image_asset_auto_migration_done_date_time``.
        customer_agreement_setting (google.ads.googleads.v21.resources.types.CustomerAgreementSetting):
            Output only. Customer Agreement Setting for a
            customer.
        local_services_settings (google.ads.googleads.v21.resources.types.LocalServicesSettings):
            Output only. Settings for Local Services
            customer.
        video_brand_safety_suitability (google.ads.googleads.v21.enums.types.BrandSafetySuitabilityEnum.BrandSafetySuitability):
            Output only. Brand Safety setting at the
            account level. Allows for selecting an inventory
            type to show your ads on content that is the
            right fit for your brand. See
            https://support.google.com/google-ads/answer/7515513.
        video_customer (google.ads.googleads.v21.resources.types.VideoCustomer):
            Video specific information about a Customer.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=19,
        optional=True,
    )
    descriptive_name: str = proto.Field(
        proto.STRING,
        number=20,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=21,
        optional=True,
    )
    time_zone: str = proto.Field(
        proto.STRING,
        number=22,
        optional=True,
    )
    tracking_url_template: str = proto.Field(
        proto.STRING,
        number=23,
        optional=True,
    )
    final_url_suffix: str = proto.Field(
        proto.STRING,
        number=24,
        optional=True,
    )
    auto_tagging_enabled: bool = proto.Field(
        proto.BOOL,
        number=25,
        optional=True,
    )
    has_partners_badge: bool = proto.Field(
        proto.BOOL,
        number=26,
        optional=True,
    )
    manager: bool = proto.Field(
        proto.BOOL,
        number=27,
        optional=True,
    )
    test_account: bool = proto.Field(
        proto.BOOL,
        number=28,
        optional=True,
    )
    call_reporting_setting: "CallReportingSetting" = proto.Field(
        proto.MESSAGE,
        number=10,
        message="CallReportingSetting",
    )
    conversion_tracking_setting: "ConversionTrackingSetting" = proto.Field(
        proto.MESSAGE,
        number=14,
        message="ConversionTrackingSetting",
    )
    remarketing_setting: "RemarketingSetting" = proto.Field(
        proto.MESSAGE,
        number=15,
        message="RemarketingSetting",
    )
    pay_per_conversion_eligibility_failure_reasons: MutableSequence[
        customer_pay_per_conversion_eligibility_failure_reason.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=16,
        enum=customer_pay_per_conversion_eligibility_failure_reason.CustomerPayPerConversionEligibilityFailureReasonEnum.CustomerPayPerConversionEligibilityFailureReason,
    )
    optimization_score: float = proto.Field(
        proto.DOUBLE,
        number=29,
        optional=True,
    )
    optimization_score_weight: float = proto.Field(
        proto.DOUBLE,
        number=30,
    )
    status: customer_status.CustomerStatusEnum.CustomerStatus = proto.Field(
        proto.ENUM,
        number=36,
        enum=customer_status.CustomerStatusEnum.CustomerStatus,
    )
    location_asset_auto_migration_done: bool = proto.Field(
        proto.BOOL,
        number=38,
        optional=True,
    )
    image_asset_auto_migration_done: bool = proto.Field(
        proto.BOOL,
        number=39,
        optional=True,
    )
    location_asset_auto_migration_done_date_time: str = proto.Field(
        proto.STRING,
        number=40,
        optional=True,
    )
    image_asset_auto_migration_done_date_time: str = proto.Field(
        proto.STRING,
        number=41,
        optional=True,
    )
    customer_agreement_setting: "CustomerAgreementSetting" = proto.Field(
        proto.MESSAGE,
        number=44,
        message="CustomerAgreementSetting",
    )
    local_services_settings: "LocalServicesSettings" = proto.Field(
        proto.MESSAGE,
        number=45,
        message="LocalServicesSettings",
    )
    video_brand_safety_suitability: (
        brand_safety_suitability.BrandSafetySuitabilityEnum.BrandSafetySuitability
    ) = proto.Field(
        proto.ENUM,
        number=46,
        enum=brand_safety_suitability.BrandSafetySuitabilityEnum.BrandSafetySuitability,
    )
    video_customer: "VideoCustomer" = proto.Field(
        proto.MESSAGE,
        number=54,
        message="VideoCustomer",
    )


class CallReportingSetting(proto.Message):
    r"""Call reporting setting for a customer. Only mutable in an ``update``
    operation.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        call_reporting_enabled (bool):
            Enable reporting of phone call events by
            redirecting them through Google System.

            This field is a member of `oneof`_ ``_call_reporting_enabled``.
        call_conversion_reporting_enabled (bool):
            Whether to enable call conversion reporting.

            This field is a member of `oneof`_ ``_call_conversion_reporting_enabled``.
        call_conversion_action (str):
            Customer-level call conversion action to attribute a call
            conversion to. If not set a default conversion action is
            used. Only in effect when call_conversion_reporting_enabled
            is set to true.

            This field is a member of `oneof`_ ``_call_conversion_action``.
    """

    call_reporting_enabled: bool = proto.Field(
        proto.BOOL,
        number=10,
        optional=True,
    )
    call_conversion_reporting_enabled: bool = proto.Field(
        proto.BOOL,
        number=11,
        optional=True,
    )
    call_conversion_action: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )


class ConversionTrackingSetting(proto.Message):
    r"""A collection of customer-wide settings related to Google Ads
    Conversion Tracking.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        conversion_tracking_id (int):
            Output only. The conversion tracking id used for this
            account. This id doesn't indicate whether the customer uses
            conversion tracking (conversion_tracking_status does). This
            field is read-only.

            This field is a member of `oneof`_ ``_conversion_tracking_id``.
        cross_account_conversion_tracking_id (int):
            Output only. The conversion tracking id of the customer's
            manager. This is set when the customer is opted into cross
            account conversion tracking, and it overrides
            conversion_tracking_id. This field can only be managed
            through the Google Ads UI. This field is read-only.

            This field is a member of `oneof`_ ``_cross_account_conversion_tracking_id``.
        accepted_customer_data_terms (bool):
            Output only. Whether the customer has
            accepted customer data terms. If using
            cross-account conversion tracking, this value is
            inherited from the manager. This field is
            read-only. For more
            information, see
            https://support.google.com/adspolicy/answer/7475709.
        conversion_tracking_status (google.ads.googleads.v21.enums.types.ConversionTrackingStatusEnum.ConversionTrackingStatus):
            Output only. Conversion tracking status. It indicates
            whether the customer is using conversion tracking, and who
            is the conversion tracking owner of this customer. If this
            customer is using cross-account conversion tracking, the
            value returned will differ based on the
            ``login-customer-id`` of the request.
        enhanced_conversions_for_leads_enabled (bool):
            Output only. Whether the customer is opted-in
            for enhanced conversions for leads. If using
            cross-account conversion tracking, this value is
            inherited from the manager. This field is
            read-only.
        google_ads_conversion_customer (str):
            The resource name of the customer where
            conversions are created and managed. This field
            is read-only.
    """

    conversion_tracking_id: int = proto.Field(
        proto.INT64,
        number=3,
        optional=True,
    )
    cross_account_conversion_tracking_id: int = proto.Field(
        proto.INT64,
        number=4,
        optional=True,
    )
    accepted_customer_data_terms: bool = proto.Field(
        proto.BOOL,
        number=5,
    )
    conversion_tracking_status: (
        conversion_tracking_status_enum.ConversionTrackingStatusEnum.ConversionTrackingStatus
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=conversion_tracking_status_enum.ConversionTrackingStatusEnum.ConversionTrackingStatus,
    )
    enhanced_conversions_for_leads_enabled: bool = proto.Field(
        proto.BOOL,
        number=7,
    )
    google_ads_conversion_customer: str = proto.Field(
        proto.STRING,
        number=8,
    )


class RemarketingSetting(proto.Message):
    r"""Remarketing setting for a customer.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        google_global_site_tag (str):
            Output only. The Google tag.

            This field is a member of `oneof`_ ``_google_global_site_tag``.
    """

    google_global_site_tag: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class CustomerAgreementSetting(proto.Message):
    r"""Customer Agreement Setting for a customer.

    Attributes:
        accepted_lead_form_terms (bool):
            Output only. Whether the customer has
            accepted lead form term of service.
    """

    accepted_lead_form_terms: bool = proto.Field(
        proto.BOOL,
        number=1,
    )


class LocalServicesSettings(proto.Message):
    r"""Settings for Local Services customer.

    Attributes:
        granular_license_statuses (MutableSequence[google.ads.googleads.v21.resources.types.GranularLicenseStatus]):
            Output only. A read-only list of geo vertical
            level license statuses.
        granular_insurance_statuses (MutableSequence[google.ads.googleads.v21.resources.types.GranularInsuranceStatus]):
            Output only. A read-only list of geo vertical
            level insurance statuses.
    """

    granular_license_statuses: MutableSequence["GranularLicenseStatus"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="GranularLicenseStatus",
        )
    )
    granular_insurance_statuses: MutableSequence["GranularInsuranceStatus"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="GranularInsuranceStatus",
        )
    )


class GranularLicenseStatus(proto.Message):
    r"""License status at geo + vertical level.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        geo_criterion_id (int):
            Output only. Geotarget criterion ID
            associated with the status. Can be on country or
            state/province geo level, depending on
            requirements and location. See
            https://developers.google.com/google-ads/api/data/geotargets
            for more information.

            This field is a member of `oneof`_ ``_geo_criterion_id``.
        category_id (str):
            Output only. Service category associated with the status.
            For example, xcat:service_area_business_plumber. For more
            details see:
            https://developers.google.com/google-ads/api/data/codes-formats#local_services_ids

            This field is a member of `oneof`_ ``_category_id``.
        verification_status (google.ads.googleads.v21.enums.types.LocalServicesVerificationStatusEnum.LocalServicesVerificationStatus):
            Output only. Granular license status, per geo
            + vertical.

            This field is a member of `oneof`_ ``_verification_status``.
    """

    geo_criterion_id: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    category_id: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    verification_status: (
        local_services_verification_status.LocalServicesVerificationStatusEnum.LocalServicesVerificationStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        optional=True,
        enum=local_services_verification_status.LocalServicesVerificationStatusEnum.LocalServicesVerificationStatus,
    )


class GranularInsuranceStatus(proto.Message):
    r"""Insurance status at geo + vertical level.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        geo_criterion_id (int):
            Output only. Geotarget criterion ID
            associated with the status. Can be on country or
            state/province geo level, depending on
            requirements and location. See
            https://developers.google.com/google-ads/api/data/geotargets
            for more information.

            This field is a member of `oneof`_ ``_geo_criterion_id``.
        category_id (str):
            Output only. Service category associated with the status.
            For example, xcat:service_area_business_plumber. For more
            details see:
            https://developers.google.com/google-ads/api/data/codes-formats#local_services_ids

            This field is a member of `oneof`_ ``_category_id``.
        verification_status (google.ads.googleads.v21.enums.types.LocalServicesVerificationStatusEnum.LocalServicesVerificationStatus):
            Output only. Granular insurance status, per
            geo + vertical.

            This field is a member of `oneof`_ ``_verification_status``.
    """

    geo_criterion_id: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    category_id: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    verification_status: (
        local_services_verification_status.LocalServicesVerificationStatusEnum.LocalServicesVerificationStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        optional=True,
        enum=local_services_verification_status.LocalServicesVerificationStatusEnum.LocalServicesVerificationStatus,
    )


class VideoCustomer(proto.Message):
    r"""Video specific information about a Customer.

    Attributes:
        third_party_integration_partners (google.ads.googleads.v21.common.types.CustomerThirdPartyIntegrationPartners):
            Third Party integration partners.
    """

    third_party_integration_partners: (
        gagc_third_party_integration_partners.CustomerThirdPartyIntegrationPartners
    ) = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gagc_third_party_integration_partners.CustomerThirdPartyIntegrationPartners,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "SummaryRowSettingEnum",
    },
)


class SummaryRowSettingEnum(proto.Message):
    r"""Indicates summary row setting in request parameter."""

    class SummaryRowSetting(proto.Enum):
        r"""Enum describing return summary row settings.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Represent unknown values of return summary
                row.
            NO_SUMMARY_ROW (2):
                Do not return summary row.
            SUMMARY_ROW_WITH_RESULTS (3):
                Return summary row along with results. The
                summary row will be returned in the last batch
                alone (last batch will contain no results).
            SUMMARY_ROW_ONLY (4):
                Return summary row only and return no
                results.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NO_SUMMARY_ROW = 2
        SUMMARY_ROW_WITH_RESULTS = 3
        SUMMARY_ROW_ONLY = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.common.types import consent as gagc_consent
from google.ads.googleads.v21.common.types import offline_user_data
from google.ads.googleads.v21.enums.types import conversion_customer_type
from google.ads.googleads.v21.enums.types import conversion_environment_enum
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "UploadClickConversionsRequest",
        "UploadClickConversionsResponse",
        "UploadCallConversionsRequest",
        "UploadCallConversionsResponse",
        "ClickConversion",
        "CallConversion",
        "ExternalAttributionData",
        "ClickConversionResult",
        "CallConversionResult",
        "CustomVariable",
        "CartData",
        "SessionAttributeKeyValuePair",
        "SessionAttributesKeyValuePairs",
    },
)


class UploadClickConversionsRequest(proto.Message):
    r"""Request message for
    [ConversionUploadService.UploadClickConversions][google.ads.googleads.v21.services.ConversionUploadService.UploadClickConversions].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        customer_id (str):
            Required. The ID of the customer performing
            the upload.
        conversions (MutableSequence[google.ads.googleads.v21.services.types.ClickConversion]):
            Required. The conversions that are being
            uploaded.
        partial_failure (bool):
            Required. If true, successful operations will
            be carried out and invalid operations will
            return errors. If false, all operations will be
            carried out in one transaction if and only if
            they are all valid. This should always be set to
            true.
            See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
        job_id (int):
            Optional. Optional input to set job ID. Must be a
            non-negative number that is less than 2^31 if provided. If
            this field is not provided, the API will generate a job ID
            in the range [2^31, (2^63)-1]. The API will return the value
            for this request in the ``job_id`` field of the
            ``UploadClickConversionsResponse``.

            This field is a member of `oneof`_ ``_job_id``.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    conversions: MutableSequence["ClickConversion"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="ClickConversion",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    job_id: int = proto.Field(
        proto.INT32,
        number=6,
        optional=True,
    )


class UploadClickConversionsResponse(proto.Message):
    r"""Response message for
    [ConversionUploadService.UploadClickConversions][google.ads.googleads.v21.services.ConversionUploadService.UploadClickConversions].

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to conversion failures in
            the partial failure mode. Returned when all
            errors occur inside the conversions. If any
            errors occur outside the conversions (for
            example, auth errors), we return an RPC level
            error. See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
        results (MutableSequence[google.ads.googleads.v21.services.types.ClickConversionResult]):
            Returned for successfully processed conversions. Proto will
            be empty for rows that received an error. Results are not
            returned when validate_only is true.
        job_id (int):
            Job ID for the upload batch.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["ClickConversionResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="ClickConversionResult",
    )
    job_id: int = proto.Field(
        proto.INT64,
        number=3,
    )


class UploadCallConversionsRequest(proto.Message):
    r"""Request message for
    [ConversionUploadService.UploadCallConversions][google.ads.googleads.v21.services.ConversionUploadService.UploadCallConversions].

    Attributes:
        customer_id (str):
            Required. The ID of the customer performing
            the upload.
        conversions (MutableSequence[google.ads.googleads.v21.services.types.CallConversion]):
            Required. The conversions that are being
            uploaded.
        partial_failure (bool):
            Required. If true, successful operations will
            be carried out and invalid operations will
            return errors. If false, all operations will be
            carried out in one transaction if and only if
            they are all valid. This should always be set to
            true.
            See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    conversions: MutableSequence["CallConversion"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="CallConversion",
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class UploadCallConversionsResponse(proto.Message):
    r"""Response message for
    [ConversionUploadService.UploadCallConversions][google.ads.googleads.v21.services.ConversionUploadService.UploadCallConversions].

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to conversion failures in
            the partial failure mode. Returned when all
            errors occur inside the conversions. If any
            errors occur outside the conversions (for
            example, auth errors), we return an RPC level
            error. See
            https://developers.google.com/google-ads/api/docs/best-practices/partial-failures
            for more information about partial failure.
        results (MutableSequence[google.ads.googleads.v21.services.types.CallConversionResult]):
            Returned for successfully processed conversions. Proto will
            be empty for rows that received an error. Results are not
            returned when validate_only is true.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["CallConversionResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="CallConversionResult",
    )


class ClickConversion(proto.Message):
    r"""A click conversion.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gclid (str):
            The Google click ID (gclid) associated with
            this conversion.

            This field is a member of `oneof`_ ``_gclid``.
        gbraid (str):
            The click identifier for clicks associated
            with app conversions and originating from iOS
            devices starting with iOS14.
        wbraid (str):
            The click identifier for clicks associated
            with web conversions and originating from iOS
            devices starting with iOS14.
        conversion_action (str):
            Resource name of the conversion action
            associated with this conversion. Note: Although
            this resource name consists of a customer id and
            a conversion action id, validation will ignore
            the customer id and use the conversion action id
            as the sole identifier of the conversion action.

            This field is a member of `oneof`_ ``_conversion_action``.
        conversion_date_time (str):
            The date time at which the conversion occurred. Must be
            after the click time. The timezone must be specified. The
            format is "yyyy-mm-dd hh:mm:ss+|-hh:mm", for example,
            "2019-01-01 12:32:45-08:00".

            This field is a member of `oneof`_ ``_conversion_date_time``.
        conversion_value (float):
            The value of the conversion for the
            advertiser.

            This field is a member of `oneof`_ ``_conversion_value``.
        currency_code (str):
            Currency associated with the conversion
            value. This is the ISO 4217 3-character currency
            code. For example: USD, EUR.

            This field is a member of `oneof`_ ``_currency_code``.
        order_id (str):
            The order ID associated with the conversion.
            An order id can only be used for one conversion
            per conversion action.

            This field is a member of `oneof`_ ``_order_id``.
        external_attribution_data (google.ads.googleads.v21.services.types.ExternalAttributionData):
            Additional data about externally attributed
            conversions. This field is required for
            conversions with an externally attributed
            conversion action, but should not be set
            otherwise.
        custom_variables (MutableSequence[google.ads.googleads.v21.services.types.CustomVariable]):
            The custom variables associated with this
            conversion.
        cart_data (google.ads.googleads.v21.services.types.CartData):
            The cart data associated with this
            conversion.
        user_identifiers (MutableSequence[google.ads.googleads.v21.common.types.UserIdentifier]):
            The user identifiers associated with this conversion. Only
            hashed_email and hashed_phone_number are supported for
            conversion uploads. The maximum number of user identifiers
            for each conversion is 5.
        conversion_environment (google.ads.googleads.v21.enums.types.ConversionEnvironmentEnum.ConversionEnvironment):
            The environment this conversion was recorded
            on, for example, App or Web.
        consent (google.ads.googleads.v21.common.types.Consent):
            The consent setting for the event.
        customer_type (google.ads.googleads.v21.enums.types.ConversionCustomerTypeEnum.ConversionCustomerType):
            Type of the customer associated with the
            conversion (new or returning). Accessible only
            to customers on the allow-list.
        user_ip_address (str):
            The IP address of the customer when they
            arrived on the landing page after an ad click
            but before a conversion event. This is the IP
            address of the customer's device, not the
            advertiser's server. Google Ads does not support
            IP address matching for end users in the
            European Economic Area (EEA), United Kingdom
            (UK), or Switzerland (CH). Add logic to
            conditionally exclude sharing IP addresses from
            users from these regions and ensure that you
            provide users with clear and comprehensive
            information about the data you collect on your
            sites, apps, and other properties and get
            consent where required by law or any applicable
            Google policies. See the
            https://support.google.com/google-ads/answer/2998031
            page for more details.

            This field is a member of `oneof`_ ``_user_ip_address``.
        session_attributes_encoded (bytes):
            The session attributes for the event, represented as a
            base64-encoded JSON string. The content should be generated
            by Google-provided library. To set session attributes
            individually, use session_attributes_key_value_pairs
            instead.

            This field is a member of `oneof`_ ``session_attributes``.
        session_attributes_key_value_pairs (google.ads.googleads.v21.services.types.SessionAttributesKeyValuePairs):
            The session attributes for the event,
            represented as key-value pairs.

            This field is a member of `oneof`_ ``session_attributes``.
    """

    gclid: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    gbraid: str = proto.Field(
        proto.STRING,
        number=18,
    )
    wbraid: str = proto.Field(
        proto.STRING,
        number=19,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    conversion_date_time: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    conversion_value: float = proto.Field(
        proto.DOUBLE,
        number=12,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    order_id: str = proto.Field(
        proto.STRING,
        number=14,
        optional=True,
    )
    external_attribution_data: "ExternalAttributionData" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="ExternalAttributionData",
    )
    custom_variables: MutableSequence["CustomVariable"] = proto.RepeatedField(
        proto.MESSAGE,
        number=15,
        message="CustomVariable",
    )
    cart_data: "CartData" = proto.Field(
        proto.MESSAGE,
        number=16,
        message="CartData",
    )
    user_identifiers: MutableSequence[offline_user_data.UserIdentifier] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=17,
            message=offline_user_data.UserIdentifier,
        )
    )
    conversion_environment: (
        conversion_environment_enum.ConversionEnvironmentEnum.ConversionEnvironment
    ) = proto.Field(
        proto.ENUM,
        number=20,
        enum=conversion_environment_enum.ConversionEnvironmentEnum.ConversionEnvironment,
    )
    consent: gagc_consent.Consent = proto.Field(
        proto.MESSAGE,
        number=23,
        message=gagc_consent.Consent,
    )
    customer_type: (
        conversion_customer_type.ConversionCustomerTypeEnum.ConversionCustomerType
    ) = proto.Field(
        proto.ENUM,
        number=26,
        enum=conversion_customer_type.ConversionCustomerTypeEnum.ConversionCustomerType,
    )
    user_ip_address: str = proto.Field(
        proto.STRING,
        number=27,
        optional=True,
    )
    session_attributes_encoded: bytes = proto.Field(
        proto.BYTES,
        number=24,
        oneof="session_attributes",
    )
    session_attributes_key_value_pairs: "SessionAttributesKeyValuePairs" = (
        proto.Field(
            proto.MESSAGE,
            number=25,
            oneof="session_attributes",
            message="SessionAttributesKeyValuePairs",
        )
    )


class CallConversion(proto.Message):
    r"""A call conversion.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        caller_id (str):
            The caller id from which this call was
            placed. Caller id is expected to be in E.164
            format with preceding '+' sign, for example,
            "+16502531234".

            This field is a member of `oneof`_ ``_caller_id``.
        call_start_date_time (str):
            The date time at which the call occurred. The timezone must
            be specified. The format is "yyyy-mm-dd hh:mm:ss+|-hh:mm",
            for example, "2019-01-01 12:32:45-08:00".

            This field is a member of `oneof`_ ``_call_start_date_time``.
        conversion_action (str):
            Resource name of the conversion action
            associated with this conversion. Note: Although
            this resource name consists of a customer id and
            a conversion action id, validation will ignore
            the customer id and use the conversion action id
            as the sole identifier of the conversion action.

            This field is a member of `oneof`_ ``_conversion_action``.
        conversion_date_time (str):
            The date time at which the conversion occurred. Must be
            after the call time. The timezone must be specified. The
            format is "yyyy-mm-dd hh:mm:ss+|-hh:mm", for example,
            "2019-01-01 12:32:45-08:00".

            This field is a member of `oneof`_ ``_conversion_date_time``.
        conversion_value (float):
            The value of the conversion for the
            advertiser.

            This field is a member of `oneof`_ ``_conversion_value``.
        currency_code (str):
            Currency associated with the conversion
            value. This is the ISO 4217 3-character currency
            code. For example: USD, EUR.

            This field is a member of `oneof`_ ``_currency_code``.
        custom_variables (MutableSequence[google.ads.googleads.v21.services.types.CustomVariable]):
            The custom variables associated with this
            conversion.
        consent (google.ads.googleads.v21.common.types.Consent):
            The consent setting for the event.
    """

    caller_id: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    call_start_date_time: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    conversion_date_time: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    conversion_value: float = proto.Field(
        proto.DOUBLE,
        number=11,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    custom_variables: MutableSequence["CustomVariable"] = proto.RepeatedField(
        proto.MESSAGE,
        number=13,
        message="CustomVariable",
    )
    consent: gagc_consent.Consent = proto.Field(
        proto.MESSAGE,
        number=14,
        message=gagc_consent.Consent,
    )


class ExternalAttributionData(proto.Message):
    r"""Contains additional information about externally attributed
    conversions.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        external_attribution_credit (float):
            Represents the fraction of the conversion
            that is attributed to the Google Ads click.

            This field is a member of `oneof`_ ``_external_attribution_credit``.
        external_attribution_model (str):
            Specifies the attribution model name.

            This field is a member of `oneof`_ ``_external_attribution_model``.
    """

    external_attribution_credit: float = proto.Field(
        proto.DOUBLE,
        number=3,
        optional=True,
    )
    external_attribution_model: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )


class ClickConversionResult(proto.Message):
    r"""Identifying information for a successfully processed
    ClickConversion.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        gclid (str):
            The Google Click ID (gclid) associated with
            this conversion.

            This field is a member of `oneof`_ ``_gclid``.
        gbraid (str):
            The click identifier for clicks associated
            with app conversions and originating from iOS
            devices starting with iOS14.
        wbraid (str):
            The click identifier for clicks associated
            with web conversions and originating from iOS
            devices starting with iOS14.
        conversion_action (str):
            Resource name of the conversion action
            associated with this conversion.

            This field is a member of `oneof`_ ``_conversion_action``.
        conversion_date_time (str):
            The date time at which the conversion occurred. The format
            is "yyyy-mm-dd hh:mm:ss+|-hh:mm", for example, "2019-01-01
            12:32:45-08:00".

            This field is a member of `oneof`_ ``_conversion_date_time``.
        user_identifiers (MutableSequence[google.ads.googleads.v21.common.types.UserIdentifier]):
            The user identifiers associated with this conversion. Only
            hashed_email and hashed_phone_number are supported for
            conversion uploads. The maximum number of user identifiers
            for each conversion is 5.
    """

    gclid: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    gbraid: str = proto.Field(
        proto.STRING,
        number=8,
    )
    wbraid: str = proto.Field(
        proto.STRING,
        number=9,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    conversion_date_time: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    user_identifiers: MutableSequence[offline_user_data.UserIdentifier] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=7,
            message=offline_user_data.UserIdentifier,
        )
    )


class CallConversionResult(proto.Message):
    r"""Identifying information for a successfully processed
    CallConversionUpload.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        caller_id (str):
            The caller id from which this call was
            placed. Caller id is expected to be in E.164
            format with preceding '+' sign.

            This field is a member of `oneof`_ ``_caller_id``.
        call_start_date_time (str):
            The date time at which the call occurred. The format is
            "yyyy-mm-dd hh:mm:ss+|-hh:mm", for example, "2019-01-01
            12:32:45-08:00".

            This field is a member of `oneof`_ ``_call_start_date_time``.
        conversion_action (str):
            Resource name of the conversion action
            associated with this conversion.

            This field is a member of `oneof`_ ``_conversion_action``.
        conversion_date_time (str):
            The date time at which the conversion occurred. The format
            is "yyyy-mm-dd hh:mm:ss+|-hh:mm", for example, "2019-01-01
            12:32:45-08:00".

            This field is a member of `oneof`_ ``_conversion_date_time``.
    """

    caller_id: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )
    call_start_date_time: str = proto.Field(
        proto.STRING,
        number=6,
        optional=True,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    conversion_date_time: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )


class CustomVariable(proto.Message):
    r"""A custom variable.

    Attributes:
        conversion_custom_variable (str):
            Resource name of the custom variable
            associated with this conversion. Note: Although
            this resource name consists of a customer id and
            a conversion custom variable id, validation will
            ignore the customer id and use the conversion
            custom variable id as the sole identifier of the
            conversion custom variable.
        value (str):
            The value string of this custom variable.
            The value of the custom variable should not
            contain private customer data, such as email
            addresses or phone numbers.
    """

    conversion_custom_variable: str = proto.Field(
        proto.STRING,
        number=1,
    )
    value: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CartData(proto.Message):
    r"""Contains additional information about cart data.

    Attributes:
        merchant_id (int):
            The Merchant Center ID where the items are
            uploaded.
        feed_country_code (str):
            The country code associated with the feed
            where the items are uploaded.
        feed_language_code (str):
            The language code associated with the feed
            where the items are uploaded.
        local_transaction_cost (float):
            Sum of all transaction level discounts, such
            as free shipping and coupon discounts for the
            whole cart. The currency code is the same as
            that in the ClickConversion message.
        items (MutableSequence[google.ads.googleads.v21.services.types.CartData.Item]):
            Data of the items purchased.
    """

    class Item(proto.Message):
        r"""Contains data of the items purchased.

        Attributes:
            product_id (str):
                The shopping id of the item. Must be equal to
                the Merchant Center product identifier.
            quantity (int):
                Number of items sold.
            unit_price (float):
                Unit price excluding tax, shipping, and any
                transaction level discounts. The currency code
                is the same as that in the ClickConversion
                message.
        """

        product_id: str = proto.Field(
            proto.STRING,
            number=1,
        )
        quantity: int = proto.Field(
            proto.INT32,
            number=2,
        )
        unit_price: float = proto.Field(
            proto.DOUBLE,
            number=3,
        )

    merchant_id: int = proto.Field(
        proto.INT64,
        number=6,
    )
    feed_country_code: str = proto.Field(
        proto.STRING,
        number=2,
    )
    feed_language_code: str = proto.Field(
        proto.STRING,
        number=3,
    )
    local_transaction_cost: float = proto.Field(
        proto.DOUBLE,
        number=4,
    )
    items: MutableSequence[Item] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message=Item,
    )


class SessionAttributeKeyValuePair(proto.Message):
    r"""Contains one session attribute of the conversion.

    Attributes:
        session_attribute_key (str):
            Required. The name of the session attribute.
        session_attribute_value (str):
            Required. The value of the session attribute.
    """

    session_attribute_key: str = proto.Field(
        proto.STRING,
        number=1,
    )
    session_attribute_value: str = proto.Field(
        proto.STRING,
        number=2,
    )


class SessionAttributesKeyValuePairs(proto.Message):
    r"""Contains session attributes of the conversion, represented as
    key-value pairs.

    Attributes:
        key_value_pairs (MutableSequence[google.ads.googleads.v21.services.types.SessionAttributeKeyValuePair]):
            Required. The session attributes for the
            conversion.
    """

    key_value_pairs: MutableSequence["SessionAttributeKeyValuePair"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="SessionAttributeKeyValuePair",
        )
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import conversion_action_category
from google.ads.googleads.v20.enums.types import conversion_origin


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CampaignConversionGoal",
    },
)


class CampaignConversionGoal(proto.Message):
    r"""The biddability setting for the specified campaign only for
    all conversion actions with a matching category and origin.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign conversion
            goal. Campaign conversion goal resource names have the form:

            ``customers/{customer_id}/campaignConversionGoals/{campaign_id}~{category}~{origin}``
        campaign (str):
            Immutable. The campaign with which this
            campaign conversion goal is associated.
        category (google.ads.googleads.v20.enums.types.ConversionActionCategoryEnum.ConversionActionCategory):
            The conversion category of this campaign
            conversion goal.
        origin (google.ads.googleads.v20.enums.types.ConversionOriginEnum.ConversionOrigin):
            The conversion origin of this campaign
            conversion goal.
        biddable (bool):
            The biddability of the campaign conversion
            goal.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
    )
    category: (
        conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory,
    )
    origin: conversion_origin.ConversionOriginEnum.ConversionOrigin = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=conversion_origin.ConversionOriginEnum.ConversionOrigin,
        )
    )
    biddable: bool = proto.Field(
        proto.BOOL,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
